#include <iostream>
#include <fstream>
#include<stdio.h>
#include<string.h>
#include<vector>
#include<map>
#include<utility>
#include<sstream>
#include<algorithm>
#include<set>
#include <queue>
//#include <igraph/igraph.h>
#include "/home/yhulovat/software/igraph/include/igraph/igraph.h"
#include<time.h>
#include <assert.h>
#include <getopt.h>
#include <math.h>

#ifdef _OPENMP
#include <omp.h>
#endif
#define PROGRESS_INFO 1
#define DEBUG 0

using namespace std;

enum ClusteringAlgorithm { walktrap, louvain, infomap };
static const char *ClusteringAlgorithmsNames[] = { "Walktrap", "Louvain", "Infomap"};
enum ScoringFunctionType {mod, aic, bic};
static const char *ScoringFunctionTypesNames[] = { "Modularity", "AIC", "BIC"};
ClusteringAlgorithm clustering_algorithm = walktrap;
ScoringFunctionType scoring_func_type = bic;

igraph_t* g_snapshots;

// AIC and BIC (note: here we use -(AIC|BIC), so we need to maximize it)
// total number of objects in the dynamic network
long n_objects = 0;
double max_aic_score = 1, max_bic_score = 1;
int max_aic_n_changepoints = -1, max_bic_n_changepoints = -1;

struct Segment {
    int t_start;
    int length;
    igraph_t *g_union;
    igraph_vector_t* edge_weights;
    double modularity;
    igraph_vector_t* membership;
    double log_l;

    Segment() {
        t_start = -1;
        length = 0;
        modularity = -1.0;
        this->edge_weights = new igraph_vector_t();
        this->g_union =  new igraph_t();
        this->membership = new igraph_vector_t();
        log_l = 0.0;
    }
    Segment(int t_start, int length) {
        this->t_start = t_start;
        this->length = length;
        modularity = -1.0;
        this->edge_weights = new igraph_vector_t();
        this->g_union =  new igraph_t();
        this->membership = new igraph_vector_t();
        log_l = 0.0;
    }

    ~Segment() {
        igraph_destroy(this->g_union);
        igraph_vector_destroy(this->edge_weights);
        igraph_vector_destroy(this->membership);
    }
};

Segment* create_singleton_segment(int t_start) {
    Segment *res = new Segment(t_start, 1);
    igraph_copy(res->g_union, &g_snapshots[t_start]);
    igraph_vector_init(res->edge_weights, igraph_ecount(res->g_union));
    igraph_vector_fill(res->edge_weights, 1.0);

    return res;
}

Segment** segments;
vector<string> split(const string &s, char delim) {
    vector<string> elems;
    stringstream ss(s);
    string item;
    while (std::getline(ss, item, delim)) {
        elems.push_back(item);
    }

    return elems;
}

double calculate_segment_log_likelihood(Segment *s, igraph_vector_t *membership) {
    int n_blocks = igraph_vector_max(membership) + 1;

    igraph_es_t igraph_ess_all(igraph_edgeorder_type_t order);
    igraph_es_t edge_selector = igraph_ess_all(IGRAPH_EDGEORDER_ID);
    igraph_vs_t vertex_selector = igraph_vss_all();

    // compute m_ij and n_ij
    vector<igraph_matrix_t> block_edges = vector<igraph_matrix_t>(s->length);
    vector<igraph_matrix_t> block_max_edges = vector<igraph_matrix_t>(s->length);

    #pragma omp parallel for
    for (int i_sn = 0; i_sn < s->length; ++i_sn){
        igraph_matrix_init(&block_edges[i_sn], n_blocks, n_blocks);
        //igraph_matrix_null(&block_edges[i_sn]);

        igraph_eit_t eit;
        igraph_eit_create(&g_snapshots[s->t_start + i_sn], edge_selector, &eit);
        while (!IGRAPH_EIT_END(eit)) {
            int eid = IGRAPH_EIT_GET(eit);
            int u, v;
            igraph_edge(&g_snapshots[s->t_start + i_sn], eid, &u, &v);
            int block_i = max((int)VECTOR(*membership)[u], (int)VECTOR(*membership)[v]);
            int block_j = min((int)VECTOR(*membership)[u], (int)VECTOR(*membership)[v]);
            ++MATRIX(block_edges[i_sn], block_i, block_j);
            IGRAPH_EIT_NEXT(eit);
        }

        vector<int> blocks_sizes = vector<int>(n_blocks, 0);
        igraph_vit_t vit;

        igraph_vit_create(&g_snapshots[s->t_start + i_sn], vertex_selector, &vit);
        while (!IGRAPH_VIT_END(vit)) {
            ++blocks_sizes[VECTOR(*membership)[IGRAPH_VIT_GET(vit)]];
            IGRAPH_VIT_NEXT(vit);
        }


        igraph_matrix_init(&block_max_edges[i_sn], n_blocks, n_blocks);
        //igraph_matrix_null(&block_max_edges[i_sn]);
        for (int i = 0; i < n_blocks; ++i) {
            for (int j=0; j < i; ++j) {
                MATRIX(block_max_edges[i_sn], i, j) += blocks_sizes[i] * blocks_sizes[j];
            }
            MATRIX(block_max_edges[i_sn], i, i) += blocks_sizes[i] * (blocks_sizes[i] - 1) / 2;
        }
    }

    // compute thetas
    igraph_matrix_t thetas;
    igraph_matrix_init(&thetas, n_blocks, n_blocks);
    //igraph_matrix_null(&thetas);
    for (int i = 0; i < n_blocks; ++i) {
        for (int j=0; j <= i; ++j) {
            double num=0, den=0;
            for (int i_sn=0; i_sn<s->length; ++i_sn) {
                num += MATRIX(block_edges[i_sn], i, j);
                den += MATRIX(block_max_edges[i_sn], i, j);
            }
            MATRIX(thetas, i, j) = num / den;
        }
    }

    // compute log_l
    double res = 0.0, temp;
    igraph_vector_t v_res;
    igraph_vector_init(&v_res, s->length);
    for (int i_sn=0; i_sn<s->length; ++i_sn){
        for (int i = 0; i < n_blocks; ++i) {
            for (int j = 0; j <= i; ++j) {
                double el1=0.0, el2=0.0;
                if (MATRIX(thetas, i, j) > 0)
                    VECTOR(v_res)[i_sn] += MATRIX(block_edges[i_sn], i, j) * log(MATRIX(thetas, i, j));
                if (MATRIX(thetas, i, j) < 1)
                    VECTOR(v_res)[i_sn] += (MATRIX(block_max_edges[i_sn], i, j) - MATRIX(block_edges[i_sn], i, j)) * log(1 - MATRIX(thetas, i, j));
            }
        }
    }

    res = igraph_vector_sum(&v_res);

    for (int i_sn=0; i_sn<s->length; ++i_sn){
        igraph_matrix_destroy(&block_edges[i_sn]);
        igraph_matrix_destroy(&block_max_edges[i_sn]);
    }
    igraph_matrix_destroy(&thetas);

    return res;
}


void cluster_segment(Segment *s){

    if (s->length > 1)  {

        int i_seg_vector = -1;
        igraph_vector_ptr_t seg_vector;
        igraph_vector_ptr_init(&seg_vector, 2);

        igraph_es_t igraph_ess_all(igraph_edgeorder_type_t order);
        igraph_es_t edge_selector = igraph_ess_all(IGRAPH_EDGEORDER_ID);
        igraph_vector_t *edges;
        map<int, map<int, double> > edge_map;
#if DEBUG
        int n_segments=0;
#endif
        for (int i = 0; i < s->length; ++i) {
            if (segments[s->t_start + i] == NULL) {
                continue;
            }

#if DEBUG
            ++n_segments;
#endif

            igraph_vector_ptr_set(&seg_vector, ++i_seg_vector, segments[s->t_start + i]->g_union);

            // to add individual weights
            igraph_t *g = segments[s->t_start + i]->g_union;
            igraph_vector_t *edge_weights = segments[s->t_start + i]->edge_weights;
            igraph_eit_t eit;
            igraph_eit_create(g, edge_selector, &eit);

            while (!IGRAPH_EIT_END(eit)) {
                int eid = IGRAPH_EIT_GET(eit);
                int v, u;
                igraph_edge(g, eid, &v, &u);

                edge_map[v][u] += VECTOR(*edge_weights)[eid];


                IGRAPH_EIT_NEXT(eit);
            }
        }

#if DEBUG
        // cout << "Merging " << n_segments << " segments (should be 2)\n";
        assert(n_segments==2);
#endif

        // compute graph union
        igraph_union_many(s->g_union, &seg_vector, NULL);

        igraph_vector_init(s->edge_weights, igraph_ecount(s->g_union));

        igraph_eit_t eit;
        igraph_eit_create(s->g_union, edge_selector, &eit);
        int i = -1;
        while (!IGRAPH_EIT_END(eit)) {
            int eid = IGRAPH_EIT_GET(eit);
            int v, u;
            igraph_edge(s->g_union, eid, &v, &u);

            if (edge_map[v][u] > 1) {
                int ii = 0;
                ii += 1;
            }

            VECTOR(*s->edge_weights)[++i] = edge_map[v][u];

            IGRAPH_EIT_NEXT(eit);
        }
    }


    igraph_vector_init(s->membership, igraph_vcount(s->g_union));
    igraph_vector_t modularity;
    igraph_vector_init(&modularity, 0);
    if (clustering_algorithm == infomap) {
        igraph_real_t codelength;
        igraph_community_infomap(s->g_union, s->edge_weights, NULL, 10, s->membership, &codelength);
        //igraph_modularity(s->g_union,  s->membership, &s->modularity, s->edge_weights);
    } else {
        if (clustering_algorithm == walktrap) {
            igraph_matrix_t merges;
            igraph_matrix_init(&merges, 0, 0);
            igraph_community_walktrap(s->g_union, s->edge_weights,
                                      4 /* steps */,
                                      &merges, &modularity, s->membership);
            igraph_matrix_destroy(&merges);
        } else if (clustering_algorithm == louvain) {
            igraph_community_multilevel(s->g_union, s->edge_weights, s->membership, NULL, &modularity);
        }
        //s->modularity = igraph_vector_max(&modularity);
        igraph_vector_destroy(&modularity);
    }


    double avg_sn_mod = 0.0;
    for(int i_sn=s->t_start;i_sn < s->t_start + s->length; ++i_sn) {
        double temp_mod;
        igraph_modularity(&g_snapshots[i_sn],  s->membership, &temp_mod, NULL);
        avg_sn_mod += temp_mod;
    }
    s->modularity = avg_sn_mod / s->length;

    s->log_l = calculate_segment_log_likelihood(s, s->membership);


}

Segment* merge_segments(Segment *s1, Segment *s2) {
    Segment *s_union = new Segment();
    s_union->t_start = s1->t_start;
    assert(s1->t_start + s1->length == s2->t_start);
    s_union->length = s1->length + s2->length;
    return s_union;
}



struct SegmentMerge {
    int s1_t_start;
    int s2_t_start;
    Segment *s_union;
    double score_change;

    SegmentMerge() {
        score_change = 0.0;
    }
    SegmentMerge(int s1_t_start, int s2_t_start) {
        this->s1_t_start=s1_t_start;
        this->s2_t_start=s2_t_start;
        this->s_union = merge_segments(segments[this->s1_t_start], segments[this->s2_t_start]);
#if DEBUG
        cout << "Merging segments [" << segments[this->s1_t_start]->t_start << ", "
        << (segments[this->s1_t_start]->t_start + segments[this->s1_t_start]->length - 1) << "]";
        cout << " and [" << segments[this->s2_t_start]->t_start << ", "
        << (segments[this->s2_t_start]->t_start + segments[this->s2_t_start]->length - 1) << "]";
        cout << " into [" << this->s_union->t_start << ", "
        << (this->s_union->t_start + this->s_union->length - 1) << "]\n";
#endif
    }

    bool operator<(const SegmentMerge & right) const
    {
        return score_change < right.score_change;
    }
};


void compute_segment_merge(SegmentMerge *sm) {
    cluster_segment(sm->s_union);
    if (scoring_func_type == mod) {
        sm->score_change = sm->s_union->modularity * sm->s_union->length
               - (segments[sm->s1_t_start]->modularity * segments[sm->s1_t_start]->length
                  + segments[sm->s2_t_start]->modularity * segments[sm->s2_t_start]->length);
    } else if (scoring_func_type == aic || scoring_func_type == bic) {
        double log_l_change = sm->s_union->log_l - (segments[sm->s1_t_start]->log_l + segments[sm->s2_t_start]->log_l);

        int n_clusters = igraph_vector_max(sm->s_union->membership) + 1;
        int n_clusters_before_1 = igraph_vector_max(segments[sm->s1_t_start]->membership) + 1;
        int n_clusters_before_2 = igraph_vector_max(segments[sm->s2_t_start]->membership) + 1;
        double n_params_change = n_clusters_before_1 * (n_clusters_before_1 + 1) /2
                + n_clusters_before_2 * (n_clusters_before_2 + 1) /2
                - n_clusters * (n_clusters + 1) / 2;
        if (scoring_func_type == bic) {
            n_params_change *= 0.5 * log(n_objects);
        }
        sm->score_change = log_l_change + n_params_change;
    }

}

bool check_segment_merge(SegmentMerge sm) {
    if (segments[sm.s1_t_start] == NULL || segments[sm.s2_t_start] == NULL) {
#if DEBUG
        cout << " Segment [" << sm.s_union->t_start << ", "
        << (sm.s_union->t_start + sm.s_union->length - 1) << "] is no longer active\n";
#endif
        return false;
    }
    return (sm.s_union->length == segments[sm.s1_t_start]->length + segments[sm.s2_t_start]->length);
}

string output_dir;
unsigned long k;
void save_solution() {
    int *changepoints = new int[k];
    int n_ch = -1;
    double total_mod = 0.0;

    //obtain changepoints (with network's first time point)
    for(int i=0;i<k; ++i){
        if (segments[i] != NULL) {
            changepoints[++n_ch] = segments[i]->t_start;
            total_mod += segments[i]->modularity * segments[i]->length;
        }
    }
    ++n_ch;

    int n_segments = n_ch;
    char *fname = new char[256];
    sprintf(fname, "%s/%d.txt", output_dir.c_str(), n_segments-1);
    FILE *fp=fopen(fname, "w");
    double avg_mod = total_mod / k;

    double total_log_l = 0.0;
    int n_params = 0;
    for(int i=0;i<k; ++i){
        if (segments[i] != NULL) {
            total_log_l += segments[i]->log_l;
            int n_clusters = igraph_vector_max(segments[i]->membership) + 1;
            n_params += n_clusters * (n_clusters + 1) / 2;
        }
    }

    double avg_log_l = total_log_l / k;
    double aic_score = total_log_l - n_params;
    double bic_score = total_log_l - 0.5 * n_params * log(n_objects);

    if (max_aic_score == 1 || aic_score > max_aic_score) {
        max_aic_score = aic_score;
        max_aic_n_changepoints = n_segments - 1;
    }

    if (max_bic_score == 1 || bic_score > max_bic_score) {
        max_bic_score = bic_score;
        max_bic_n_changepoints = n_segments - 1;
    }


#if DEBUG
    cout << "Segments - " << n_segments << ", score - " << avg_mod << ", avg log_l - " << avg_log_l << ", bic - " << bic << ", changepoints - ";
    for(int i=1;i<n_ch; ++i){
        cout << changepoints[i] << " ";
    }
    cout << "\n";
#endif

    fprintf(fp, "params=6\nstart_timepoint=0\nend_timepoint=%d\n",k-1);
    fprintf(fp, "avg_mod=%.9f\n", avg_mod);
    fprintf(fp, "avg_log_l=%.9f\n", avg_log_l);
    fprintf(fp, "aic=%.9f\n", aic_score);
    fprintf(fp, "bic=%.9f\n", bic_score);
    fprintf(fp, "changepoints=");
    for(int i=1;i<n_ch; ++i){
        fprintf(fp, "%d,", changepoints[i]);
    }
    fprintf(fp, "\npartitions_count=%d\n", n_segments);
    for (int i=0; i<n_segments;  ++i) {
        Segment *curr = segments[changepoints[i]];
        fprintf(fp, "partition [%d, %d]\n", curr->t_start, curr->t_start + curr->length - 1);
        int n_nodes = igraph_vector_size(curr->membership);
        fprintf(fp, "%d\n", n_nodes);
        for (int j=0; j<n_nodes; ++j ) {
            fprintf(fp, "%d,%d\n", j, (int)VECTOR(*curr->membership)[j]);
        }
    }
    fclose(fp);

    delete(fname);
    delete(changepoints);

}

void copy_solution(ScoringFunctionType ic){
    assert(ic == aic || ic == bic);
    char *fname_src = new char[256];
    fprintf(stderr, "Solution with max %s has %d changepoint(s)\n", ic == aic ? "AIC" : "BIC", max_aic_n_changepoints);
    sprintf(fname_src, "%s/%d.txt", output_dir.c_str(), ic == aic ? max_aic_n_changepoints : max_bic_n_changepoints);
    std::ifstream  src(fname_src, std::ios::binary);
    char *fname = new char[256];
    sprintf(fname, "%s/_%s.txt", output_dir.c_str(), ic == aic ? "aic" : "bic");
    std::ofstream  dst(fname,   std::ios::binary);
    dst << src.rdbuf();

}

int main(int argc, char *argv[]){
    // measure running time
    time_t start, end;
    time(&start);

    // location of snapshots
    string snapshots_files_str;
    string file_format = "edge_list";

    if(argc<2) {
        cout << "ERROR: Incorrect number of input parameters\n";
        cout << "USAGE: " << argv[0] << "<input_files> <output_dir> -c <clustering_method> -s <cscd_objective_function> -h help\n";
        return 1;
    }


    int c;
    string temp_str;
    while ((c = getopt(argc, argv, "c:s:h")) != -1){
        switch (c){
            case 'c':
                temp_str = optarg;
                if (temp_str == "walktrap")
                    clustering_algorithm = walktrap;
                else if (temp_str == "louvain")
                    clustering_algorithm = louvain;
                else if (temp_str == "infomap")
                    clustering_algorithm = infomap;
                else {
                    cout << "Unknown clustering algorithm " << optarg << " (allowed values: [walktrap|louvain|infomap])\n";
                    return 0;
                }
                break;
            case 's':
                temp_str = optarg;
                if (temp_str == "mod")
                    scoring_func_type = mod;
                else if (temp_str == "aic")
                    scoring_func_type = aic;
                else if (temp_str == "bic")
                    scoring_func_type = bic;
                else {
                    cout << "Unknown objective function " << optarg << " (allowed values: [mod|aic|bic])\n";
                    return 0;
                }
                break;
            case 'h':
                cout << "<input_files> <output_dir> -h help\n";
                return 0;
            default:
                abort();
        }
    }

    snapshots_files_str = argv[optind];
    output_dir = argv[optind+1];


    fprintf(stderr, "Using static clustering method '%s' and scoring method '%s'\n",
            ClusteringAlgorithmsNames[clustering_algorithm], ScoringFunctionTypesNames[scoring_func_type]
    );


#ifdef _OPENMP
    int nProcessors = omp_get_max_threads();
	fprintf(stderr, "%d processors available\n", nProcessors);
	omp_set_num_threads(nProcessors);
#endif

    vector<string> snapshots_files = split(snapshots_files_str, ',');
    k = snapshots_files.size();

    g_snapshots = new igraph_t[k];
    #pragma omp parallel for
    for(int i=0;i<k;++i) {
        FILE *instream = fopen(snapshots_files[i].c_str(), "r");
        if (file_format == "edge_list") {
            igraph_read_graph_edgelist(&g_snapshots[i], instream, 0, false);
        } else if (file_format == "glg") {
            igraph_read_graph_ncol(&g_snapshots[i], instream, NULL, false, IGRAPH_ADD_WEIGHTS_IF_PRESENT, false);
        }
        fclose(instream);
        igraph_simplify(&g_snapshots[i], 0 /* multiple */, 1 /* loop */, /* edge_comb= */ 0 );
    }

    // total number of objects in the dynamic network - needed for BIC
    n_objects = 0;
    for(int i=0;i<k;++i) {
        int n_count = igraph_vcount(&g_snapshots[i]);
        n_objects += n_count * (n_count - 1) / 2;
    }

    segments = new Segment*[k];
    #pragma omp parallel for
    for(int i=0;i<k;++i) {
        segments[i] = create_singleton_segment(i);
        cluster_segment(segments[i]);
    }

    priority_queue<SegmentMerge>  *pq = new priority_queue<SegmentMerge>;

    //compute initial merges
    #pragma omp parallel for
    for(int i=0;i<k-1;++i){
        SegmentMerge sm = SegmentMerge(i, i+1);
        compute_segment_merge(&sm);
        pq->push(sm);
    }
    //save solution with current segmentation (i.e., k singleton segments)
    save_solution();

    int n_merges = 0;
    while(pq->size() > 0) {
        SegmentMerge curr = pq->top();
        pq->pop();

#if DEBUG
        cout << " Trying to make new segment - [" << curr.s_union->t_start << ", "
        << (curr.s_union->t_start + curr.s_union->length - 1) << "]\n";
#endif

        //first, check if the segment merge is still valid
        if (check_segment_merge(curr)) {
#if PROGRESS_INFO
            time(&end);
            fprintf(stderr, "\rRemaining merges -  %d  [%d s elapsed]", k - (++n_merges) - 1, end - start);
#endif
            int s1_t_start = curr.s1_t_start;
            int s2_t_start = curr.s2_t_start;

            delete (segments[s1_t_start]);
            delete (segments[s2_t_start]);
            segments[s1_t_start] = curr.s_union;
            segments[s2_t_start] = NULL;
            save_solution();

            //get previous segment and merge with s_union
            int i_s_prev = -1;
            for (int i = s1_t_start - 1; i >= 0; --i) {
                if (segments[i] != NULL) {
                    i_s_prev = i;
                    break;
                }
            }

            //get next segment and merge with s_union
            int i_s_next = -1;
            for (int i = s2_t_start + 1; i < k; ++i) {
                if (segments[i] != NULL) {
                    i_s_next = i;
                    break;
                }
            }

            SegmentMerge *sm_prev = NULL;
            SegmentMerge *sm_next = NULL;
            #pragma omp parallel sections num_threads(2)
            {
                #pragma omp section
                if (i_s_prev >= 0) {
                    sm_prev =  new SegmentMerge(i_s_prev, s1_t_start);
                    compute_segment_merge(sm_prev);

                }

                #pragma omp section
                if (i_s_next >= 0) {
                    sm_next = new SegmentMerge(s1_t_start, i_s_next);
                    compute_segment_merge(sm_next);
                }
            }
            if (sm_prev != NULL) {
                pq->push(*sm_prev);
            }
            if (sm_next != NULL) {
                pq->push(*sm_next);
            }
        }
    }

    fprintf(stderr, "\n");

    copy_solution(aic);
    copy_solution(bic);

    //delete[] g_snapshots;
    //delete[] segments;

    time(&end);
    double diff = difftime(end, start);
    fprintf(stderr, "Total running time: %.f seconds\n", diff);

    return 0;
}