import os, shutil

from src.core.log import logger
from src.core.utils import parse_user_args_string_into_values_list, get_subdir_list_from_dir
from src.core.method import Method
from src.core import settings
from src.core import report_params


def backup_synthetic_networks(n_ids):
    """
    Backup networks (without methods states)
    :param n_ids:
    :return:
    """
    for n_id in parse_user_args_string_into_values_list(n_ids):
        s_n_id = n_id.split('/')[-1]
        n_segments = s_n_id.split('.')[1]
        temp = s_n_id.split('.')[3].split('_')
        rand_instance = temp[0]
        n_nodes = temp[1]
        n_dir = settings.TMP_DIR + "_public/synthetic_networks/%s_nodes/%s_segment%s/%s/" % (n_nodes, n_segments, "s" if int(n_segments)!=1 else "", rand_instance)
        if not os.path.exists(n_dir):
            os.makedirs(n_dir)
        shutil.copytree(settings.NETWORKS_BASE_DIR + n_id + '/snapshots/', n_dir + '/snapshots/')
        shutil.copy(settings.NETWORKS_BASE_DIR + n_id + '/ground_truth/segment_ground_truth/data.txt', n_dir + "ground_truth.txt")


def backup_methods_states(methods_ids, networks_ids):
    methods_ids = []
    for m_key in ['naive-ic', 'naive-obj', 'naive-cons-sum', 'naive-cons', 'naive-search']:
        for temp_m in report_params.REPORT_METHODS_BY_KEY[m_key]:
            methods_ids += parse_user_args_string_into_values_list(temp_m)
    methods_ids = ['naive-u^w^walktrap-(bic)-bu']
    methods_ids = list(set(methods_ids)) # remove duplicates
    for n_id in networks_ids:
        m_dir = settings.TMP_DIR + "_backup/" + n_id + "/"
        if not os.path.exists(m_dir):
            os.makedirs(m_dir)
        for m_id in methods_ids:
            in_dir = Method.get_method_output_dir(n_id, m_id)
            last_modified = os.path.getmtime(in_dir + '/state.cpickle')
            # if time.time() - last_modified > 4 * 24 * 3600:
            #    raise Exception("Modified long time ago: %s" % time.ctime(os.path.getmtime(in_dir + '/state.cpickle')))

            if not os.path.exists(in_dir):
                logger.warning(in_dir)
                continue

            shutil.copytree(in_dir, m_dir + m_id)

def load_methods_states_from_backup():
    n_prefix = 'synthetic/'
    backup_dir = settings.TMP_DIR + '_naive_backup_1/'
    backup_dir_2 = settings.TMP_DIR + '_naive_backup_2/'
    # copy methods from backup_dir to the main dir,
    #  but if they already exist in the main dir, backup the current before overwrite to backup_dir_2
    # (if (backup_dir_2 is not None)
    for n_suffix in get_subdir_list_from_dir(backup_dir + n_prefix, full_path=False):
        n_id = n_prefix + n_suffix
        for m_id in get_subdir_list_from_dir(backup_dir + n_id + '/', full_path=False):
            logger.info("Processing %s - %s" % (n_id, m_id))
            m_dir = Method.get_method_output_dir(n_id, m_id)

            if os.path.exists(m_dir):
                if backup_dir_2 is not None:
                    shutil.move(m_dir, backup_dir_2 + n_id + '/' + m_id + '/')
                else:
                    shutil.rmtree(m_dir)
            shutil.copytree(backup_dir + n_id + '/' + m_id + '/', m_dir)