import numpy as np
import math
from ..com_evs_similarity_method import ComEvsSimilarityMethod
from ..partition import Partition
from ..affinity_matrix import AffinityMatrix

class AvgSnapshotPartitionSimilarity(ComEvsSimilarityMethod):
    """
    Compare community evolution  by comparing their partitions at each timepoint
    """

    def __init__(self, partition_similarity_metric='ami'):
        self._partition_similarity_metric = partition_similarity_metric

    def get_id(self):
        return '({prefix})({partition_similarity_metric})'.format(
            prefix=AvgSnapshotPartitionSimilarity.get_id_prefix(),
            partition_similarity_metric=self.get_partition_similarity_metric(),
        )

    @staticmethod
    def get_id_prefix():
        return 'avg_sn_p'

    @staticmethod
    def get_com_evs_similarity_method_title(values):
        return "$Sim_P$ ({partition_similarity_metric})".format(
            partition_similarity_metric=values[1].upper().upper() #Partition.SIMILARITY_METRICS.get(values[1], AffinityMatrix.PARTITION_SIMILARITY_METRICS.get(values[1]))
        )

    def get_partition_similarity_metric(self):
        return self._partition_similarity_metric

    def compute_similarity(self, com_ev_1, com_ev_2):
        if len(com_ev_1.get_segments_partitions()) == 0 or len(com_ev_2.get_segments_partitions()) == 0:
            return None
        timepoints = com_ev_1.get_dynamic_network().get_timepoints()
        # choose similarity function to handle both partition-vs-partition and partition-vs-matrix
        is_aff = isinstance(com_ev_2.get_segments_partitions()[0], AffinityMatrix)
        if (is_aff and self.get_partition_similarity_metric() not in AffinityMatrix.PARTITION_SIMILARITY_METRICS) or\
            (not is_aff and self.get_partition_similarity_metric() not in Partition.SIMILARITY_METRICS):
            return None
        similarities = []
        for i in timepoints:
            similarities.append(
                Partition.compute_partitions_similarity(
                    com_ev_2.get_snapshot_partition(i),
                    com_ev_1.get_snapshot_partition(i),
                    metric=self.get_partition_similarity_metric(),
                ) if not is_aff else
                AffinityMatrix.compute_partition_internal_quality(
                    partition=com_ev_1.get_snapshot_partition(i),
                    affinity_matrix=com_ev_2.get_snapshot_partition(i),
                    metric=self.get_partition_similarity_metric()
                )
            )
        return np.average([s for s in similarities if (s is not None and not math.isnan(s))])

