from ..com_evs_similarity_method import ComEvsSimilarityMethod
from ..partition import Partition

class GlobalPartitionSimilarity(ComEvsSimilarityMethod):
    """
    Compare community evolution  by comparing their partitions
    Treat each node in each snapshot as a separate object and then compare overall partitions similarity
    """

    def __init__(self, partition_similarity_metric='ami'):
        self._partition_similarity_metric = partition_similarity_metric

    def get_id(self):
        return '({prefix})({partition_similarity_metric})'.format(
            prefix=GlobalPartitionSimilarity.get_id_prefix(),
            partition_similarity_metric=self.get_partition_similarity_metric(),
        )

    @staticmethod
    def get_id_prefix():
        return 'gl_p'

    @staticmethod
    def get_com_evs_similarity_method_title(values):
        return "$Sim_B$ ({partition_similarity_metric})".format(
            partition_similarity_metric=values[1].upper() # Partition.SIMILARITY_METRICS[values[1]].upper()
        )

    def get_partition_similarity_metric(self):
        return self._partition_similarity_metric

    def compute_similarity(self, com_ev_1, com_ev_2):
        if not com_ev_1.has_partitions() or not com_ev_2.has_partitions():
            return None
        obj_clusters_1 = {}
        obj_clusters_2 = {}
        timepoints = com_ev_1.get_dynamic_network().get_timepoints()
        for i in timepoints:
            p1 = com_ev_1.get_snapshot_partition(i)
            s1 = com_ev_1.get_snapshot_segment(i)
            for obj, cl in p1.get_objects_clusters().iteritems():
                obj_clusters_1['%s-%s' % (i, obj)] =  '%s-%s' % (s1, cl)

            p2 = com_ev_2.get_snapshot_partition(i)
            s2 = com_ev_2.get_snapshot_segment(i)
            for obj, cl in p2.get_objects_clusters().iteritems():
                obj_clusters_2['%s-%s' % (i, obj)] = '%s-%s' % (s2, cl)

        return  Partition.compute_partitions_similarity(
            partition_true=Partition(obj_clusters_2),
            partition_predicted=Partition(obj_clusters_1),
            metric=self.get_partition_similarity_metric(),
        )
