import numpy as np

from ..com_evs_similarity_method import ComEvsSimilarityMethod
from ..partition import Partition

class AvgSegSimilarity(ComEvsSimilarityMethod):
    """
    Compare community evolution by comparing their time segmentations
    """

    def __init__(self, partition_similarity_metric='ami'):
        self._partition_similarity_metric = partition_similarity_metric

    def get_id(self):
        return '({prefix})({partition_similarity_metric})'.format(
            prefix=AvgSegSimilarity.get_id_prefix(),
            partition_similarity_metric=self.get_partition_similarity_metric(),
        )

    @staticmethod
    def get_id_prefix():
        return 'avg_seg_t'

    @staticmethod
    def get_com_evs_similarity_method_title(values):
        return "$Sim_T$ ({partition_similarity_metric})".format(
            partition_similarity_metric=values[1].upper() # Partition.SIMILARITY_METRICS[values[1]].upper()
        )

    def get_partition_similarity_metric(self):
        return self._partition_similarity_metric

    def compute_similarity(self, com_ev_1, com_ev_2):
        timepoints = com_ev_1.get_dynamic_network().get_timepoints()
        p = []
        for com_ev in [com_ev_1, com_ev_2]:
            obj_clusters = [0] * len(timepoints)
            for t in timepoints:
                obj_clusters[t] = com_ev.get_snapshot_segment(t)
            p.append(Partition(obj_clusters))

        return Partition.compute_partitions_similarity(
            partition_true=p[1],
            partition_predicted=p[0],
            metric=self.get_partition_similarity_metric(),
        )


