import os
from collections import OrderedDict

from .. import settings
from ..log import logger
from ..affinity_matrix import AffinityMatrix
from .. import method
import sankey
import draw_utils
import partition_draw
from .. import static_clustering
from ..com_evs_similarity_methods import com_evs_similarity_methods_utils
from ..scoring_methods import scoring_methods_utils
import corr_draw
import numpy as np

def draw_list_partitions_internal_qualities_timeseries(c_evs_list, c_ev_gt, output_dir,
                                                       dyn_network=None, overwrite=True):
    logger.debug("Drawing time-series of partition internal qualities with respect to {c_ev_gt} per timepoint".format(
        c_ev_gt=c_ev_gt.get_method_title(),
    ))
    dn = dyn_network or c_evs_list[0].get_dynamic_network()
    output_dir += 'timeseries-snapshot-partitions_similarity/'

    vertical_lines = None
    if dn.get_segment_ground_truth() is not None:
        vertical_lines=[
            {
                'title': "ground truth",
                'x_list': [t - 0.5 for t in dn.get_segment_ground_truth().get_changepoints()],
                'y_text_relative': 0.2,
                'fontsize': 10,
            }
        ]
    for metric in settings.DEFAULT_PARTITION_SIMILARITY_AFFINITY_METRICS:
        metric_title = AffinityMatrix.PARTITION_SIMILARITY_METRICS[metric]
        if os.path.exists(output_dir + metric + '.' + settings.CHART_EXT):
            logger.debug('Timeseries of partition internal qualities has been already drawn: %s'
                         % output_dir + metric + '.' + settings.CHART_EXT)
            if not overwrite:
                logger.debug("No overwrite: aborting")
                continue
        data = OrderedDict()
        for i in xrange(len(dn.get_timepoints())):
            p_gt = c_ev_gt.get_snapshot_partition(i)
            if p_gt is None:
                continue
            for c_ev in c_evs_list:
                p_c = c_ev.get_snapshot_partition(i)
                if p_c is None:
                    continue
                if c_ev.get_method_title() not in data:
                    data[c_ev.get_method_title()] = []
                data[c_ev.get_method_title()].append(
                    (
                        i,
                        AffinityMatrix.compute_partition_internal_quality(p_c, p_gt, metric, use_common_objects=True)
                    )
                )
        if len(data) > 0:
            draw_utils.draw_timeseries(
                data=data,
                output_dir=output_dir,
                output_filename=metric,
                x_label="Snapshot",
                y_label=metric_title,
                y_lim=AffinityMatrix.get_similarity_metric_limits(metric),
                # output_data_dir=images_dir + 'images_data/',
                vertical_lines=vertical_lines,
            )