import os
from .. import utils
from ..network import Network
from ..log import logger
import draw_utils
import network_draw

def draw_dyn_network(dyn_network, overwrite=True):
    logger.debug("Drawing dynamic network '{network_id}'".format(network_id=dyn_network.get_id()))
    draw_snapshots_properties(dyn_network, overwrite=overwrite)
    if utils.is_remote_server():
        draw_snapshots(dyn_network, overwrite=overwrite)

def draw_snapshots(dynamic_network, overwrite=True):
    logger.debug("Drawing snapshots for '{network_id}'".format(network_id=dynamic_network.get_id()))
    output_dir = dynamic_network.get_images_dir() + 'networks/'
    if os.path.exists(output_dir):
        logger.debug('Snapshots have been already drawn: %s' % output_dir)
        if not overwrite:
            logger.debug("No overwrite: aborting")
            return

    node_positions = None
    i_segment_start = 0
    for i in xrange(len(dynamic_network.get_snapshots())):
        #  network from the same segment will use the same node positions
        if i_segment_start < len(dynamic_network.get_segment_ground_truth().get_changepoints()) and\
                        i == dynamic_network.get_segment_ground_truth().get_changepoints()[i_segment_start]:
            node_positions = None
            i_segment_start += 1

        if dynamic_network.get_snapshots()[i].get_nodes_count() > 100:
            logger.warning(
                "Snapshot is too big for drawing: number of %s > 100"
                % dynamic_network.get_snapshots()[i].get_nodes_count()
            )
            continue

        node_positions, image_file = network_draw.draw_network(
            network=dynamic_network.get_snapshots()[i],
            output_dir=output_dir,
            node_positions = node_positions,
            output_filename='%s' % i,
            nodes_colors_dict=dynamic_network.get_segment_ground_truth().get_snapshot_partition(i).get_objects_clusters()
                if dynamic_network.get_segment_ground_truth() else None,
        )

def draw_snapshots_properties(dynamic_network, overwrite=True):
    logger.debug("Drawing time-series of snapshot properties")
    output_dir = dynamic_network.get_images_dir() + 'network_properties/'
    if os.path.exists(output_dir):
        logger.debug('Time-series of snapshot properties have been already drawn: %s' % output_dir)
        if not overwrite:
            logger.debug("No overwrite: aborting")
            return

    vertical_lines = None
    if dynamic_network.get_segment_ground_truth() is not None:
        vertical_lines = [
            {
                'title': "changepoint",
                'x_list': [dynamic_network.get_segment_ground_truth().get_changepoints()[i] - 0.5],
                'y_text_relative': 0.2,
                'fontsize': 10,
            } for i in xrange(len(dynamic_network.get_segment_ground_truth().get_changepoints()))
        ]

    for prop, prop_title in Network.NETWORK_PROPERTIES.iteritems():
        logger.debug("Drawing property %s" % prop_title)
        draw_utils.draw_timeseries(
            data={
                dynamic_network._network_id: [
                    (i, snapshot.get_network_property(prop))
                    for i, snapshot in enumerate(dynamic_network.get_snapshots())
                ]
            },
            output_dir=output_dir,
            output_filename='timeseries-snapshot-%s' % prop,
            x_label="Snapshot",
            y_label=prop_title,
            y_lim=(0, None),
            show_legend=False,
            output_data_dir=dynamic_network.get_images_dir() + 'network_properties/images_data/',
            vertical_lines=vertical_lines,
        )