from ..log import logger
from ..draw import com_ev_draw
from collections import OrderedDict
from ..methods import methods_utils
from ..community_evolution import CommunityEvolution

def evaluate_com_ev_against_other_across_networks_dict(method_id, networks_dict, other_method_id, output_dir,
                                                   **kwargs):
    logger.debug("Evaluating methods {method_id} across networks '{networks_dict}'  against {gt}".format(
        method_id=method_id,
        networks_dict=", ".join(networks_dict),
        gt=other_method_id,
    ))

    output_dir += 'evaluation_against_other/' + other_method_id + '/'

    if kwargs.get('c_evs_dict') is not None:
        c_evs_dict = kwargs.get('c_evs_dict')
        kwargs.pop('c_evs_dict')
    else:
        c_evs_dict = OrderedDict()
        for n_key, networks_ids in networks_dict.iteritems():
            c_evs = []
            for n_id in networks_ids:
                c_evs.append(CommunityEvolution.load(n_id, method_id))
            c_evs_dict[n_key] = c_evs

    c_evs_gts_dict = OrderedDict()
    for n_key, networks_ids in networks_dict.iteritems():
        c_evs = []
        for n_id in networks_ids:
            c_evs.append(CommunityEvolution.load(n_id, other_method_id))
        c_evs_gts_dict[n_key] = c_evs

    com_ev_draw.draw_list_com_evs_similarities_to_other_bars_avg(
        c_evs_dict=c_evs_dict,
        c_evs_gts_dict=c_evs_gts_dict,
        output_dir=output_dir,
        com_evs_similarity_methods_ids=None,
        draw_pairwise_corr_matrices=True,
        **kwargs
    )

    com_ev_draw.draw_list_com_evs_similarities_to_other_scatter_avg(
        c_evs_dict=c_evs_dict,
        c_evs_gts_dict=c_evs_gts_dict,
        output_dir=output_dir,
        **kwargs
    )

def evaluate_com_evs_against_other_across_networks(methods_ids, networks_ids, other_method_id, output_dir,
                                                   **kwargs):
    logger.debug("Evaluating methods {methods_ids} across networks '{networks_ids}'  against {gt}".format(
        methods_ids=", ".join(methods_ids),
        networks_ids=", ".join(networks_ids),
        gt=other_method_id,
    ))

    output_dir += 'evaluation_against_other/' + other_method_id + '/'
    if kwargs.get('c_evs_dict') is not None:
        c_evs_dict = kwargs.get('c_evs_dict')
        kwargs.pop('c_evs_dict')
    else:
        c_evs_dict = OrderedDict()
        for m_id in methods_ids:
            c_evs = []
            for n_id in networks_ids:
                c_evs.append(CommunityEvolution.load(n_id, m_id))
            c_evs_dict[methods_utils.get_method_title(m_id)] = c_evs

    c_evs_gts_list = []
    for n_id in networks_ids:
        c_evs_gts_list.append(CommunityEvolution.load(n_id, other_method_id))

    com_ev_draw.draw_list_com_evs_similarities_to_other_bars_avg(
        c_evs_dict=c_evs_dict,
        c_evs_gts_list=c_evs_gts_list,
        output_dir=output_dir,
        com_evs_similarity_methods_ids=None,
        **kwargs
    )

    com_ev_draw.draw_list_com_evs_similarities_to_other_scatter_avg(
        c_evs_dict=c_evs_dict,
        c_evs_gts_list=c_evs_gts_list,
        output_dir=output_dir,
        **kwargs
    )

def evaluate_com_evs_against_other_across_networks_dict(methods_ids, networks_dict, other_method_id, output_dir,
                                                   **kwargs):
    logger.debug("Evaluating methods {methods_ids} across networks '{networks_dict}'  against {gt}".format(
        methods_ids=", ".join(methods_ids),
        networks_dict=", ".join(networks_dict.keys()),
        gt=other_method_id,
    ))

    if kwargs.get('add_subdirs', True):
        output_dir += 'evaluation_against_other/' + other_method_id + '/'

    if kwargs.get('networks_c_evs_dict') is not None:
        networks_c_evs_dict = kwargs.get('networks_c_evs_dict')
        kwargs.pop('networks_c_evs_dict')
    else:
        networks_c_evs_dict = OrderedDict()
        for n_key, networks_ids in networks_dict.iteritems():
            networks_c_evs_dict[n_key] = OrderedDict()
            for m_id in methods_ids:
                c_evs = []
                for n_id in networks_ids:
                    c_evs.append(CommunityEvolution.load(n_id, m_id))
                networks_c_evs_dict[n_key][methods_utils.get_method_title(m_id)] = c_evs

    networks_c_evs_gts_list = OrderedDict()
    for n_key, networks_ids in networks_dict.iteritems():
        networks_c_evs_gts_list[n_key] = []
        for n_id in networks_ids:
            networks_c_evs_gts_list[n_key].append(CommunityEvolution.load(n_id, other_method_id))

    com_ev_draw.draw_dict_networks_com_evs_similarities_to_other_bars_avg(
        networks_c_evs_dict=networks_c_evs_dict,
        networks_c_evs_gts_list=networks_c_evs_gts_list,
        output_dir=output_dir,
        **kwargs
    )
