import os
from .. import utils
from ..dynamic_network import DynamicNetwork

def generate_methods_comparison_key(methods_ids_list):
    """
    When comparing multiple methods, we need to save results into a corresponding subdirectory.
    This function generates a corresponding subdirectory name.
    :param methods_ids_list:
    :return:
    """
    if len("".join(methods_ids_list)) < 70:
        comparison_methods_hash = ("-vs-".join(sorted(methods_ids_list))).replace(' ','_')
    else:
        comparison_methods_hash = str(len(methods_ids_list)) + '_'\
                         + utils.hash_string("|".join(sorted(methods_ids_list)), size=10)
    return comparison_methods_hash

def generate_networks_comparison_key(networks_ids):
    """
    When comparing across several networks, we need to save results into a corresponding subdirectory.
    This function generates a corresponding subdirectory name.
    :param methods_ids_list:
    :return:
    """

    if isinstance(networks_ids, dict):
        networks_ids_str = "-".join(["%s|%s" % (k, ",".join(sorted(networks_ids[k]))) for k in sorted(networks_ids.keys())])
    else:
        networks_ids_str = "-".join(sorted(networks_ids))

    if len(networks_ids_str) < 70:
        comparison_networks_hash = (networks_ids_str).replace(' ','_')
    else:
        comparison_networks_hash = str(len(networks_ids)) + '_' + utils.hash_string(networks_ids_str, size=10)
    return comparison_networks_hash


def get_netwoks_list_comparison_key(networks_ids_list, **kwargs):
    if kwargs.get('networks_list_key_name_in_dict') is not None and kwargs.get('networks_key_name') is not None:
        return '%s-%s' % (kwargs.get('networks_key_name'), kwargs.get('networks_list_key_name_in_dict'))
    else:
        return generate_networks_comparison_key(networks_ids_list)


def init_comparison_dir(dynamic_network_id, methods_ids_list, comparison_methods_key=None):
    comparison_methods_key = comparison_methods_key or generate_methods_comparison_key(methods_ids_list)
    output_dir = DynamicNetwork.get_network_methods_comparison_dir(dynamic_network_id)\
        + comparison_methods_key + '/'

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
        with open(output_dir + 'methods.txt','wb') as outfile:
            outfile.writelines("\n".join(methods_ids_list))
    return output_dir