import sys
import argparse
import numpy as np
import networkx as nx

from ..log import logger, log_func_progress
from ..dynamic_network import DynamicNetwork
from ..utils import parse_user_args_string_into_values_list, get_subdir_list_from_dir
from ..evaluation import method_eval
from ..method import Method
from ..evaluation import com_ev_eval, ranking_eval, stats_eval
from ..draw import com_ev_draw, method_draw, dyn_network_draw,  draw_utils, adhoc_draw, paper_figures_draw
from .. import settings
from ..scoring_methods import scoring_methods_utils
from ..com_evs_similarity_methods import com_evs_similarity_methods_utils
from ..partition import Partition
from ..evaluation import method_eval_across_networks
from ..evaluation import method_eval
from collections import OrderedDict
from ..community_evolution import CommunityEvolution
from .. import report_params
from ..classification_result import ClassificationResult
from ..evaluation import eval_utils, adhoc_eval
from ..methods import methods_utils

def compute_stats_significance_of_improvement(m_key='best', n_key='all-synthetic-configs'):
    """
    Compute statistical significance of SCOUT's improvement over others
    :return:
    """

    methods_dict = {}
    for temp_m in report_params.REPORT_METHODS_BY_KEY[m_key]:
        for m_id in parse_user_args_string_into_values_list(temp_m):
            m_key = 'naive' if methods_utils.is_SCOUT_method_id(m_id) else 'others'
            if m_key not in methods_dict:
                methods_dict[m_key] = []
            methods_dict[m_key].append(m_id)

    networks_dict = report_params.get_networks_dict_by_key(n_key)

    stats_eval.evalute_methods_score_p_vals(
        methods_dict=methods_dict,
        networks_dict=networks_dict,
        scoring_method_ids=[
            '(com_evs_sim)((avg_seg_t)(nmi))(segment_ground_truth)', # Sim_T
            '(avg_sn)(mod)',  # Q_P
            '(com_evs_sim)((avg_sn_p)(nmi))(segment_ground_truth)', # Sim_P
            '(com_evs_sim)((gl_p)(nmi))(segment_ground_truth)', # Sim_B
            '(cl_res)(aupr)(segment_ground_truth)'  # AUPR
        ],
        output_dir=settings.EVAL_DIR + '_stats/' + m_key\
                      + '/' + n_key + '/'
    )

def compute_rankings():
    methods_dict = {
        'graphscope': [],
        'multistep': [],
        'ghrg': [],
        'naive': [],
    }

    def m_func(m_key, dn):
        if m_key == 'graphscope':
            return ['graphscope']
        if m_key == 'multistep':
            # if len(dn.get_segment_ground_truth().get_changepoints()) in [0, 15]:
            #     return []
            # no need for others, since here we use ground truth number of segments -- advantage to Multi-Step!
            return ["multistep-3.0"] #[m_id for m_id in dn.get_computed_methods_ids_list() if 'multistep' in m_id]
        if m_key == 'ghrg':
            #return []
            return [m_id for m_id in dn.get_computed_methods_ids_list() if 'ghrg-4' in m_id ]
        if m_key == 'naive':
            return ['cppnaive-walktrap-bic']
            # return ['naive-u^w^walktrap-(bic)-bu', 'cppnaive-walktrap-bic']# + ['naive-u^w^louvain-(bic)-bu', 'cppnaive-louvain-bic']
            # return [m_id for m_id in dn.get_computed_methods_ids_list() if 'naive' in m_id]
        return None


    m_comparison_key = eval_utils.generate_methods_comparison_key(methods_dict.keys())

    scoring_methods_ids_dict ={
        'all_configs': [
            '(com_evs_sim)((avg_seg_t)(nmi))(segment_ground_truth)', # Sim_T
            '(avg_sn)(mod)',  # Q_P
            '(com_evs_sim)((avg_sn_p)(nmi))(segment_ground_truth)', # Sim_P
            '(com_evs_sim)((gl_p)(nmi))(segment_ground_truth)', # Sim_B
        ],
        'non_extreme_configs': [
            '(cl_res)(aupr)(segment_ground_truth)' # AUPR
        ],
    }

    for configs_type, scoring_methods_ids in scoring_methods_ids_dict.iteritems():
        n_key = "small-synthetic-configs%s" % ("" if configs_type != 'non_extreme_configs' else "-nonextreme")
        networks_dict = report_params.get_networks_dict_by_key(n_key)
        ranking_eval.evalute_methods_score_rankings_per_key(
            methods_dict=methods_dict,
            networks_dict=networks_dict,
            scoring_method_ids=scoring_methods_ids,
            precision_levels=[0, 0.05, 0.1, 0.15, 0.2, 0.25],
            m_func=m_func, # to get methods ids for a method key
            output_dir=settings.EVAL_DIR + '_rankings/' + m_comparison_key\
                          + '/' + n_key + '/'
        )



        # ranking_eval.evalute_methods_score_rankings(
        #     methods_dict=methods_dict,
        #     networks_ids=networks_ids,
        #     scoring_method_ids=scoring_methods_ids,
        #     precision_levels=[0, 0.05, 0.1, 0.15, 0.2, 0.25],
        #     m_func=m_func, # to get methods ids for a method key
        #     output_dir=settings.EVAL_DIR + '_rankings_m_best_cpp/' + m_comparison_key\
        #                   + '/' + eval_utils.generate_networks_comparison_key(networks_ids) + '/'
        # )
        #
        # methods_dict={"multistep": [ "multistep-0.0"], "naive": ["naive-u^w^walktrap-(avg_sn)(mod)-bu", "cppnaive-walktrap-mod-bic"]}
        # ranking_eval.evalute_methods_score_rankings_n_changepoints(
        #     methods_dict=methods_dict,
        #     networks_ids=networks_ids,
        #     scoring_method_ids=['(avg_sn)(mod)'],
        #     n_timepoints=16,
        #     precision_levels=[0],#, 0.05, 0.1, 0.15, 0.2, 0.25],
        #     # m_func=m_func, # to get methods ids for a method key
        #     output_dir=settings.EVAL_DIR + '_rankings_m_best_mod/' + eval_utils.generate_methods_comparison_key(methods_dict.keys())\
        #                   + '/' + eval_utils.generate_networks_comparison_key(networks_ids) + '/'
        # )
        #