from naive import NaiveMethod
from naive_cpp import NaiveCppMethod
from other_methods.multistep import MultistepMethod
from other_methods.graphscope import GraphScope
from other_methods.ghrg import GHRG
from .. import settings

SEGMENT_GROUND_TRUTH_METHOD_ID = 'segment_ground_truth'
METHODS_CLASSES = [
    NaiveMethod,
    NaiveCppMethod,
    MultistepMethod,
    GraphScope,
    GHRG,
]
SCOUT_METHODS_CLASSES = [
    NaiveMethod,
    NaiveCppMethod,
]
def create_method_by_id(method_id):
    """
    Create method object by id
    :param method_id:
    :return: Method
    """
    values = method_id.split(settings.METHOD_ID_SEPARATOR)
    for cl in METHODS_CLASSES:
        if values[0] == cl.get_id_prefix():
            return cl(method_id)

    raise Exception("Unknown method id: %s" % method_id)

def get_method_title(method_id):
    if method_id == SEGMENT_GROUND_TRUTH_METHOD_ID:
        return "Segment ground truth"
    else:
        values = method_id.split(settings.METHOD_ID_SEPARATOR)

        for cl in METHODS_CLASSES:
            if values[0] == cl.get_id_prefix():
                return cl.get_method_title(method_id)

    return method_id

def get_method_description(method_id):
    values = method_id.split(settings.METHOD_ID_SEPARATOR)
    for cl in METHODS_CLASSES:
        if values[0] == cl.get_id_prefix():
            return cl.get_method_description(method_id)

    return get_method_title(method_id)

def is_SCOUT_method_id(method_id):
    values = method_id.split(settings.METHOD_ID_SEPARATOR)
    for cl in SCOUT_METHODS_CLASSES:
        if values[0] == cl.get_id_prefix():
            return True
    return False