import multiprocessing
from log import logger, log_func_progress, close_filehandler
import time
import settings

class CustomProcess(multiprocessing.Process):
    def run(self):
        logger.info("Process %s started" % self.name)

        if self._target:
            self._target(*self._args, **self._kwargs)
            # target_with_log = log_func_progress(self._target)
            # target_with_log(*self._args, **self._kwargs)

        logger.info("Process {process_name} finished".format(process_name=self.name))
        # TODO: currently in remote server all processes write to the same file - init in setup_logger() for children processes is not called
        # rename log file according to _target exit status
        # close_filehandler()


def run_process_pool(processes_list,
                     max_processes_count=settings.MAX_PROCESSES_COUNT,
                     sec_to_wait=5):
    curr_running = 0
    logger.info("Running process pool of size {size} with {max_count} max process count: {processes}.".format(
        size=len(processes_list),
        max_count=max_processes_count,
        processes=", ".join(p.name for p in processes_list)
    ))
    for i_p, p in enumerate(processes_list):
        while curr_running >= max_processes_count:
            logger.warning(
                "{curr} >= {max} processes are already running. "
                "Waiting for {sec_to_wait} sec before attempting to start the {remaining}/{total} remaining.".format(
                    curr=curr_running,
                    max=max_processes_count,
                    sec_to_wait=sec_to_wait,
                    remaining=len(processes_list) - i_p,
                    total=len(processes_list),
                )
            )
            time.sleep(sec_to_wait)
            curr_running = len(multiprocessing.active_children())
        p.start()
        curr_running += 1
    logger.info(
        "All processes from the pool has been started. Waiting for %s to finish."
        % len(multiprocessing.active_children())
    )
    for p in processes_list:
        p.join()
    logger.info("Finished running process pool")
