import preprocess_utils
from .. import settings
from ..community_evolution import CommunityEvolution
from ..log import logger
from ..partition import Partition
from pytz import timezone
from ..affinity_matrix import AffinityMatrix

def create_dynamic_network(delta_t='1week', threshold=1):
    network_sub_name = '%s_%s' % (delta_t, threshold)
    network_id = 'high_school/' + network_sub_name
    network_dir = settings.NETWORKS_BASE_DIR + network_id + '/'

    timepoints_labels = preprocess_utils.split_into_snapshots(
        input_temporal_network_file=settings.NETWORKS_RAW_DATA_DIR + '/high_school_2013/High-School_data_2013.csv',
        output_dir_snapshots=network_dir + 'snapshots/',
        delta_t=delta_t,
        threshold=threshold,
        delimiter=' ',
        min_t=1385982020, # 12/02/2013
        as_directed_graph=False,
        as_weighted_graph=True,
        min_nodes_snapshot=5,
        min_edges_snapshot=5,
        save_snapshots_names_ordered=True,
        tz=timezone('Europe/Paris'),
    )
    timepoints_datetimes = preprocess_utils.convert_timepoint_labels_to_dt(timepoints_labels, delta_t)

    # ground truth partitions
    people_class = {}
    people_gender = {}
    node_labels = {}
    with open(settings.NETWORKS_RAW_DATA_DIR + 'high_school_2013/metadata_2013.txt', 'r') as infile:
        for l in infile.readlines():
            person_id,class_name,gender = l.strip().split('\t')
            people_class[person_id] = class_name
            if gender != 'Unknown':
                people_gender[person_id] = gender
            node_labels[person_id] = "%s-%s" % (class_name, gender)

    gt_partitions = {
        'class': Partition(people_class),
        'gender': Partition(people_gender),
    }

    friendship_types = {
        'fb': 'Facebook-known-pairs_data_2013.csv',
        'friends': 'Friendship-network_data_2013.csv'
    }
    for fr_name, fr_file in friendship_types.iteritems():
        affinities = []
        logger.info("Processing friendship info from %s" % fr_file)
        with open(settings.NETWORKS_RAW_DATA_DIR + 'high_school_2013/%s' % fr_file, 'r') as infile:
            for l in infile.readlines():
                values = l.strip().split(' ')
                i = values[0]
                j = values[1]
                are_friends = 1.0 if (values[2] == '1' if len(values)==3 else True) else 0.0
                affinities.append((i,j,are_friends))
        gt_partitions[fr_name] = AffinityMatrix.from_affinities_list(affinities)

    for gt_title, gt_partition in gt_partitions.iteritems():
        c_ev = CommunityEvolution(
            segments_partitions=[gt_partition],
            changepoints=None,
            method_id='gt_%s' % gt_title,
            network_id=network_id,
            start_timepoint=0,
            end_timepoint=len(timepoints_labels)-1,
        )
        c_ev.save(as_ground_truth=True)


    # save node labels
    preprocess_utils.save_node_labels(network_dir, node_labels)

    # save description
    with open(settings.NETWORKS_BASE_DIR + network_id + '/description.txt', 'wb') as outfile:
        outfile.write(
            "This is High School contact network. The raw data was obtained from here http://www.sociopatterns.org/datasets/high-school-contact-and-friendship-networks/. "
            "Each snapshot contains data of {delta_t} and was constructed with threshold {threshold}".format(
                delta_t=delta_t,
                threshold=threshold
            )
        )

