import preprocess_utils
import datetime
from .. import settings
import bisect
from ..community_evolution import CommunityEvolution
from ..log import logger
from ..partition import Partition
from collections import defaultdict

def create_dynamic_network(delta_t='1week', threshold=1):
    network_sub_name = '%s_%s' % (delta_t, threshold)
    network_id = 'hypertext/' + network_sub_name
    network_dir = settings.NETWORKS_BASE_DIR + network_id + '/'

    timepoints_labels = preprocess_utils.split_into_snapshots(
        input_temporal_network_file=settings.NETWORKS_RAW_DATA_DIR + '/hypertext_2009/ht09_contact_list.dat',
        output_dir_snapshots=network_dir + 'snapshots/',
        delta_t=delta_t,
        threshold=threshold,
        delimiter='\t',
        min_t=20,
        as_directed_graph=False,
        as_weighted_graph=True,
        min_nodes_snapshot=5,
        min_edges_snapshot=5,
        t_convert_func=lambda t: datetime.datetime.fromtimestamp(int(t) + 1246255200 + 6 * 3600), # see readme
        save_snapshots_names_ordered=True,
    )
    timepoints_datetimes = preprocess_utils.convert_timepoint_labels_to_dt(timepoints_labels, delta_t)

    # load changepoints - events-only ground truth
    c_ev = CommunityEvolution(
        segments_partitions=None,
        changepoints=preprocess_utils.load_changepoints(
            settings.NETWORKS_RAW_DATA_DIR + 'hypertext_2009/events.txt', timepoints_datetimes,
            datetime_format='%m/%d/%Y %I:%M %p',
        ),
        method_id='gt_events',
        network_id=network_id,
        start_timepoint=0,
        end_timepoint=len(timepoints_labels)-1,
    )
    c_ev.save(as_ground_truth=True)


    # save description
    with open(network_dir + 'description.txt', 'wb') as outfile:
        outfile.write(
            "This is Hypertext network. The raw data was obtained from here http://www.sociopatterns.org/datasets/hypertext-2009-dynamic-contact-network/. "
            "Each snapshot contains data of {delta_t} and was constructed with threshold {threshold}".format(
                delta_t=delta_t,
                threshold=threshold
            )
        )

