import requests, json, time, os

def get_json_values(file_name):
    res = {}
    result_records = json.loads(open(file_name, "r").read())["results"]
    for r in result_records:
        res[r["id"]] = r["text"].encode('utf8')
    return res

partner_areas = get_json_values("parameters_ids/partnerAreas.json")
reporter_areas = get_json_values("parameters_ids/reporterAreas.json")
trade_regimes = get_json_values("parameters_ids/tradeRegimes.json")

classification = "S1"
commodities = get_json_values("parameters_ids/classification{classification}.json".format(
        classification=classification
    )
)

# records for which no data is available - to avoid extra requests
missing_data_log_filename=  "missing_data_log.txt"
missing_data_list = []
if os.path.exists(missing_data_log_filename):
    missing_data_list = open(missing_data_log_filename, "r").readlines()

# http://comtrade.un.org/data/doc/api/
parameters = {
    "fmt": "csv",
    "max": 50000,
    "type": "C", # trade data type (C - Commodities (merchandise trade data))
    "head": "M",
    "frequency": "A", # annual or monthly
    "px": classification,
    "p": "all",
    "rg": "1,2", # trade regimes (1 - Import, 2 - Export)
    "cc": "TOTAL", # a commodity code valid in the selected classification
}
years = map(str,range(1962, 2015))
reporters = [str(r) for r in reporter_areas.keys() if r != "all"]
i_year = 0
i_reporter = 0
# aggregate requests to reduce the number of requests
year_aggregate = 1
reporter_aggregate = 5
while i_year < len(years):
    i_year_next = i_year + year_aggregate
    parameters["ps"] = ",".join(years[i_year:min(i_year_next, len(years))])
    i_year = i_year_next
    out_dir = "../../../input/world_trade_networks/raw_data/{cc}/{year}/".format(
        cc=parameters["cc"].lower(),
        year=parameters["ps"].replace(",","_"),
    )
    if not os.path.exists(out_dir):
        os.makedirs(out_dir)

    #while i_reporter < len(reporters):
        # i_reporter_next = (i_reporter + reporter_aggregate
        # parameters["r"] = ",".join(reporters[i_reporter:max(i_reporter_next,, len(reporters)-1)])
        # i_reporter = i_reporter_next
    parameters["r"] = "" # hack to get data for all

    out_filename = "wtn_{reporter}_{partner}_{regime}.csv".format(
        reporter=parameters["r"].replace(",","-") if parameters["r"]!="" else "all",
        partner=partner_areas[parameters["p"]].replace(".","").lower(),
        regime=parameters["rg"].replace(",","-").lower(),
    ).replace(" ","-")
    descr = "reporter='{reporter}' and partner='{partner}' for year={year} and regime='{regime}'".format(
            reporter=parameters["r"].replace(",",", ") if parameters["r"]!="" else "All (with hack)",
            partner=partner_areas[parameters["p"]],
            year=parameters["ps"].replace(",",", "),
            regime=parameters["rg"].replace(",",", "),
    )

    if os.path.exists(os.path.join(out_dir, out_filename)):
        print "Data for \"{descr}\" already exists".format(
            descr=descr,
        )
        continue

    if descr in missing_data_list:
        print "No data for \"{descr}\" : tried to access before".format(
            descr=descr,
        )
        continue

    retries = 2
    while retries >= 0:
        # rate limit is 1 second
        time.sleep(1)

        r = requests.get(
            "http://comtrade.un.org/api/get",
            params=parameters
        )
        if r.reason == "Conflict":
            print "Conflict for {descr}: {message}".format(
                descr=descr,
                message=r.text,
            )
            print "Sleeping for one hour..."
            time.sleep(3600)
            retries -= 1
        else:
            break

    if "too complex" in r.text:
        print "No data for {descr}".format(
            descr=descr,
        )
        missing_data_out_file = open(missing_data_log_filename,"a")
        missing_data_out_file.write(descr + "\n")
        missing_data_out_file.close()
        continue
    elif r.reason == "Conflict":
        print "Conflict for {descr} even after retries: {message}".format(
            descr=descr,
            message=r.text,
        )
        continue

    print "Got data for {descr}: {count} records".format(
        descr=descr,
        count=r.text.count("\n"),
    )

    with open(os.path.join(out_dir, out_filename), "wb") as outfile:
        outfile.write(r.text.encode('utf8'))
