import preprocess_utils
import datetime
from .. import settings
import bisect
from ..community_evolution import CommunityEvolution
from ..log import logger
from ..partition import Partition
from collections import defaultdict
import networkx as nx
from ..network import Network
import ast
import numpy as np

def create_dynamic_network(delta_t='1week', threshold=0):
    network_sub_name = '%s_%s' % (delta_t, threshold)
    network_id = 'voteview/' + network_sub_name
    network_dir = settings.NETWORKS_BASE_DIR + network_id + '/'

    t_aggr, t_unit = preprocess_utils.get_time_unit_type_from_delta_t(delta_t)
    if t_unit != 'year':
        raise Exception("Unknown delta_t %s" % delta_t)

    state_code_to_name  = {}
    with open(settings.NETWORKS_RAW_DATA_DIR + 'voteview/state_codes_icpsr.txt','r') as infile:
        for line in infile.readlines():
            values = line.split()
            state_code = int(values[0])
            state_name = line[6:].strip().lower().replace(' ', '_')
            state_code_to_name[state_code] = state_name

    senator_to_state = {}
    with open(settings.NETWORKS_RAW_DATA_DIR + 'voteview/senate_icpsr.txt','r') as infile:
        for line in infile.readlines():
            values = line.split()
            senator_id = values[1]
            state_code = int(values[2])
            senator_to_state[senator_id] = state_code

    congress_to_years = ast.literal_eval(open(settings.NETWORKS_RAW_DATA_DIR + 'voteview/congress_to_years.txt','r').read())
    max_i = 114
    i = 1
    snapshots_names = []
    while i < max_i:
        temp_networks = []
        for j in xrange(i, min(i + t_aggr, max_i)):
            temp_networks.append(
                Network.load(
                    input_path=settings.NETWORKS_RAW_DATA_DIR + 'voteview/senate_rollcall_by_state/%s.ncol' % i,
                    read_method='edgelist',
                    data=(('weight',float),),
                )
            )
        if t_aggr > 1:
            g_union = Network.graph_union(temp_networks, weighted=True)
        else:
            g_union = temp_networks[0]

        nx_g_res = nx.Graph()
        for e in g_union.get_nx_graph().edges(data=True):
            if e[2]['weight'] < threshold:
                continue
            nx_g_res.add_edge(e[0], e[1], weight=e[2]['weight'])

        if nx_g_res.size() == 0:
            logger.warning("No edges for %s" % congress_to_years[i])
        else:
            snapshots_names.append(str(congress_to_years[i]))
            g_res = Network(nx_g_res)
            g_res.save(
                output_dir=network_dir + 'snapshots/',
                output_filename=snapshots_names[-1],
            )

        i += t_aggr

    timepoints_datetimes = preprocess_utils.convert_timepoint_labels_to_dt(snapshots_names, delta_t)
    # load changepoints - events-only ground truth
    # c_ev = CommunityEvolution(
    #     segments_partitions=None,
    #     changepoints=preprocess_utils.load_changepoints(
    #         settings.NETWORKS_RAW_DATA_DIR + 'voteview/events.txt', timepoints_datetimes,
    #         datetime_format='%Y',
    #     ),
    #     method_id='gt_events',
    #     network_id=network_id,
    #     start_timepoint=0,
    #     end_timepoint=len(snapshots_names)-1,
    # )
    # c_ev.save(as_ground_truth=True)

    # save ordered snapshots names
    preprocess_utils.save_snapshots_names_ordered(
        output_dir=network_dir,
        snapshots_names=snapshots_names,
    )

    # save node labels
    preprocess_utils.save_node_labels(network_dir, state_code_to_name)

    # save description
    with open(settings.NETWORKS_BASE_DIR + network_id + '/description.txt', 'wb') as outfile:
        outfile.write(
            "This is Vote View contact network. The raw data was obtained from here https://github.com/kawadia/estrangement/tree/master/data/voteview. "
            "Each snapshot contains data of {delta_t} and was constructed with threshold {threshold}".format(
                delta_t=delta_t,
                threshold=threshold
            )
        )

