from collections import OrderedDict
from utils import parse_user_args_string_into_values_list

def get_networks_dict_by_key(n_key):
    networks_dict = OrderedDict()
    for n_k, n_ids_str in report_params.REPORT_NETWORKS_DICT_BY_KEY.get(n_key).iteritems():
        networks_dict[n_k] = parse_user_args_string_into_values_list(n_ids_str)
    return networks_dict


# METHODS KEYS
# each key contains a list of methods (for e.g. 'xxx[1-10]xxx' - they are unrolled into a list)
# so, these can be used for comparing certain different methods or different parameters of the same method
REPORT_METHODS_BY_KEY = {
    'other': ['graphscope',  'multistep', 'ghrg-4'],
    'ghrg': ['ghrg-[4|8|12]'],
    'multistep': ['multistep-[0.0|0.2|0.4|0.6|0.8|1.0]'],
    'best': [
        'graphscope',
        'multistep-3.0', # Multi-Step*
        'ghrg-4',
        'cppnaive-walktrap-bic',
    ],
    'best-mod': [
        'graphscope',
        'multistep-3.0',  # Multi-Step*
        'ghrg-4',
        'cppnaive-walktrap-mod-bic',
    ],
    'best-m': [
        'graphscope',
        'multistep-0.0', # Multistep - 0.0
        'ghrg-4',
        'cppnaive-walktrap-bic',
        # 'cppnaive-walktrap-bic-serial', # serial version
    ],
    'best-n-changepoints': [
        'graphscope',
        'multistep-0.0',
        'ghrg-4',
        'cppnaive-walktrap-mod-bic',
    ],
    'test-n-changepoints': [
        'graphscope',
        'multistep-0.0',
        'ghrg-4',
        'cppnaive-walktrap-bic',
        'cppnaive-walktrap-mod-bic',
         'naive-u^w^[walktrap]-(bic)-bu',
         'naive-u^w^[walktrap]-(avg_sn)(mod)-bu',
    ],
    'naive-ic': [
        'naive-u^w^walktrap-([aic|bic])-exh',
    ],
    'naive-obj': [
       'naive-u^w^walktrap-(avg_sn)([mod|cond_q|norm_cut_q|avg_odf_q])-exh',
       'naive-u^w^walktrap-(bic)-exh',
    ],
    'naive-cons-sum': [
       'naive-u^w^[fastgreedy|label_prop|lead_eigenv|infomap|walktrap|louvain|stab_louvain]-(bic)-exh',
    ],
    'naive-cons': [
       'naive-u^w^[walktrap]-(bic)-exh',
       'naive-avg_mod-(bic)-exh',
       'naive-cons_matr^[walktrap]-(bic)-exh'
    ],
    'naive-search': [
        'naive-u^w^walktrap-(bic)-[exh|td|bu]',
    ],
    'naive-bu': [
       'naive-avg_mod-(avg_sn)(mod)-bu',
       'naive-u^w^[fastgreedy|label_prop|lead_eigenv|infomap|walktrap|louvain]-(avg_sn)(mod)-bu',
    ],
    'naive-td': [
       'naive-avg_mod-(avg_sn)(mod)-td',
       'naive-u^w^[fastgreedy|label_prop|lead_eigenv|infomap|walktrap|louvain]-(avg_sn)(mod)-td',
    ],
    'naive-exh': [
       'naive-avg_mod-(avg_sn)(mod)-exh',
       'naive-u^w^[fastgreedy|label_prop|lead_eigenv|infomap|walktrap|louvain|stab_louvain]-(avg_sn)(mod)-exh',
    ],
}


# NETWORKS KEYS
# similar to the above, but for networks
# e.g., can be used when need to run something for a list of networks
REPORT_NETWORKS_DICT_BY_KEY = {
    'all-synthetic': OrderedDict([
        ('50', 'synthetic_16_50/sbm-rand.[1|2|4|8|16].16.[1-10]_50_0-20-4-0-0'),
        ('100', 'synthetic/sbm-rand.[1|2|4|8|16].16.[1-10]_100_0-20-4-0-0'),
        ('500', 'synthetic_16_500/sbm-rand.[1|2|4|8|16].16.[1-10]_500_0-20-4-0-0'),
        ('1000', 'synthetic/sbm-rand.[1|2|4|8|16].16.[1-10]_1000_0-20-4-0-0'),
    ]),
    'all-synthetic-configs': OrderedDict([
        ('50 nodes - 1 segment', 'synthetic_16_50/sbm-rand.1.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 2 segments', 'synthetic_16_50/sbm-rand.2.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 4 segments', 'synthetic_16_50/sbm-rand.4.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 8 segments', 'synthetic_16_50/sbm-rand.8.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 16 segments', 'synthetic_16_50/sbm-rand.16.16.[1-10]_50_0-20-4-0-0'),
        ('100 nodes - 1 segment', 'synthetic/sbm-rand.1.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 2 segments', 'synthetic/sbm-rand.2.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 4 segments', 'synthetic/sbm-rand.4.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 8 segments', 'synthetic/sbm-rand.8.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 16 segments', 'synthetic/sbm-rand.16.16.[1-10]_100_0-20-4-0-0'),
        ('500 nodes - 1 segment', 'synthetic_16_500/sbm-rand.1.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 2 segments', 'synthetic_16_500/sbm-rand.2.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 4 segments', 'synthetic_16_500/sbm-rand.4.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 8 segments', 'synthetic_16_500/sbm-rand.8.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 16 segments', 'synthetic_16_500/sbm-rand.16.16.[1-10]_500_0-20-4-0-0'),
        ('1000 nodes - 1 segment', 'synthetic/sbm-rand.1.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 2 segments', 'synthetic/sbm-rand.2.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 4 segments', 'synthetic/sbm-rand.4.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 8 segments', 'synthetic/sbm-rand.8.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 16 segments', 'synthetic/sbm-rand.16.16.[1-10]_1000_0-20-4-0-0'),
     ]),
    'small-synthetic-configs': OrderedDict([
        ('50 nodes - 1 segment', 'synthetic_16_50/sbm-rand.1.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 2 segments', 'synthetic_16_50/sbm-rand.2.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 4 segments', 'synthetic_16_50/sbm-rand.4.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 8 segments', 'synthetic_16_50/sbm-rand.8.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 16 segments', 'synthetic_16_50/sbm-rand.16.16.[1-10]_50_0-20-4-0-0'),
        ('100 nodes - 1 segment', 'synthetic/sbm-rand.1.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 2 segments', 'synthetic/sbm-rand.2.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 4 segments', 'synthetic/sbm-rand.4.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 8 segments', 'synthetic/sbm-rand.8.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 16 segments', 'synthetic/sbm-rand.16.16.[1-10]_100_0-20-4-0-0'),
    ]),
    'small-synthetic-configs-nonextreme': OrderedDict([
        ('50 nodes - 2 segments', 'synthetic_16_50/sbm-rand.2.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 4 segments', 'synthetic_16_50/sbm-rand.4.16.[1-10]_50_0-20-4-0-0'),
        ('50 nodes - 8 segments', 'synthetic_16_50/sbm-rand.8.16.[1-10]_50_0-20-4-0-0'),
        ('100 nodes - 2 segments', 'synthetic/sbm-rand.2.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 4 segments', 'synthetic/sbm-rand.4.16.[1-10]_100_0-20-4-0-0'),
        ('100 nodes - 8 segments', 'synthetic/sbm-rand.8.16.[1-10]_100_0-20-4-0-0'),
    ]),
    'large-synthetic-configs': OrderedDict([
        ('500 nodes - 1 segment', 'synthetic_16_500/sbm-rand.1.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 2 segments', 'synthetic_16_500/sbm-rand.2.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 4 segments', 'synthetic_16_500/sbm-rand.4.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 8 segments', 'synthetic_16_500/sbm-rand.8.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 16 segments', 'synthetic_16_500/sbm-rand.16.16.[1-10]_500_0-20-4-0-0'),
        ('1000 nodes - 1 segment', 'synthetic/sbm-rand.1.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 2 segments', 'synthetic/sbm-rand.2.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 4 segments', 'synthetic/sbm-rand.4.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 8 segments', 'synthetic/sbm-rand.8.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 16 segments', 'synthetic/sbm-rand.16.16.[1-10]_1000_0-20-4-0-0'),
    ]),
    'large-synthetic-configs-nonextreme': OrderedDict([
        ('500 nodes - 2 segments', 'synthetic_16_500/sbm-rand.2.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 4 segments', 'synthetic_16_500/sbm-rand.4.16.[1-10]_500_0-20-4-0-0'),
        ('500 nodes - 8 segments', 'synthetic_16_500/sbm-rand.8.16.[1-10]_500_0-20-4-0-0'),
        ('1000 nodes - 2 segments', 'synthetic/sbm-rand.2.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 4 segments', 'synthetic/sbm-rand.4.16.[1-10]_1000_0-20-4-0-0'),
        ('1000 nodes - 8 segments', 'synthetic/sbm-rand.8.16.[1-10]_1000_0-20-4-0-0'),
    ]),
    'rnd16n50': OrderedDict([
        ('1 segment', 'synthetic_16_50/sbm-rand.1.16.[1-10]_50_0-20-4-0-0'),
        ('2 segments', 'synthetic_16_50/sbm-rand.2.16.[1-10]_50_0-20-4-0-0'),
        ('4 segments', 'synthetic_16_50/sbm-rand.4.16.[1-10]_50_0-20-4-0-0'),
        ('8 segments', 'synthetic_16_50/sbm-rand.8.16.[1-10]_50_0-20-4-0-0'),
        ('16 segments', 'synthetic_16_50/sbm-rand.16.16.[1-10]_50_0-20-4-0-0'),
    ]),
    'rnd16n100': OrderedDict([
        ('1 segment', 'synthetic/sbm-rand.1.16.[1-10]_100_0-20-4-0-0'),
        ('2 segments', 'synthetic/sbm-rand.2.16.[1-10]_100_0-20-4-0-0'),
        ('4 segments', 'synthetic/sbm-rand.4.16.[1-10]_100_0-20-4-0-0'),
        ('8 segments', 'synthetic/sbm-rand.8.16.[1-10]_100_0-20-4-0-0'),
        ('16 segments', 'synthetic/sbm-rand.16.16.[1-10]_100_0-20-4-0-0'),
    ]),
    'rnd16n500': OrderedDict([
        ('1 segment', 'synthetic_16_500/sbm-rand.1.16.[1-10]_500_0-20-4-0-0'),
        ('2 segments', 'synthetic_16_500/sbm-rand.2.16.[1-10]_500_0-20-4-0-0'),
        ('4 segments', 'synthetic_16_500/sbm-rand.4.16.[1-10]_500_0-20-4-0-0'),
        ('8 segments', 'synthetic_16_500/sbm-rand.8.16.[1-10]_500_0-20-4-0-0'),
        ('16 segments', 'synthetic_16_500/sbm-rand.16.16.[1-10]_500_0-20-4-0-0'),
    ]),
    'rnd16n1000': OrderedDict([
        ('1 segment', 'synthetic/sbm-rand.1.16.[1-10]_1000_0-20-4-0-0'),
        ('2 segments', 'synthetic/sbm-rand.2.16.[1-10]_1000_0-20-4-0-0'),
        ('4 segments', 'synthetic/sbm-rand.4.16.[1-10]_1000_0-20-4-0-0'),
        ('8 segments', 'synthetic/sbm-rand.8.16.[1-10]_1000_0-20-4-0-0'),
        ('16 segments', 'synthetic/sbm-rand.16.16.[1-10]_1000_0-20-4-0-0'),
    ]),
    'all-realworld': OrderedDict([
        ('Hypertext', 'hypertext/30min_1'),
        ('AMD Hope', 'amd_hope/coloc/1.0_1hour_1'),
        ('High School', 'high_school/1hour_1'),
        ('Reality Mining', 'reality_mining/calls/1week_1'),
        ('Enron', 'enron/1month_1'),
        ('Senate', 'voteview/1year_0.4'),
     ]),
    'realitymining': OrderedDict([
        ('weekly', 'reality_mining/calls/1week_1'),
      #  ('bi-weekly', 'reality_mining/calls/2week_1'),
     ]),
    'fa': OrderedDict([
        #('def 1 year', 'cow/fa/def_1year_1'),
        ('def 5 year', 'cow/fa/def_5year_1'),
        #('def 10 year', 'cow/fa/def_10year_1'),
        #('all 1 year', 'cow/fa/all_1year_1'),
        #('all 5 year', 'cow/fa/all_5year_1'),
        #('all 10 year', 'cow/fa/all_10year_1'),
     ]),
    'enron': OrderedDict([
        ('weekly', 'enron/1week_1'),
        #('bi-weekly', 'enron/2week_1'),
        #('monthly', 'enron/1month_1'),
     ]),
    'hypertext': OrderedDict([
        ('30 min', 'hypertext/30min_1'),
        #('1 hour', 'hypertext/1hour_1'),
       # ('2 hours', 'hypertext/2hour_1'),
     ]),
    'highschool': OrderedDict([
        # ('30 min', 'high_school/30min_1'),
        ('1 hour', 'high_school/1hour_1'),
        ('2 hours', 'high_school/2hour_1'),
     ]),
    'amdhope': OrderedDict([
        # ('30 min', 'high_school/30min_1'),
        ('1 hour', 'amd_hope/coloc/1.0_1hour_1'),
       # ('2 hours', 'amd_hope/attend/18_0_2hour_1'),
     ]),
}