import os
import time
import shutil
from ..log import logger
from .. import settings
from .. import utils


def get_methods_networks_key(methods_ids, networks_ids, **kwargs):
   return 'm_%s_%s-n_%s_%s' % (
        len(methods_ids), kwargs.get('methods_key_name') or utils.hash_string("|".join(methods_ids), size=5),
        len(networks_ids),kwargs.get('networks_key_name') or  utils.hash_string("|".join(networks_ids), size=5)
    )

def copy_report_to_tmp_dir(report, tmp_subdir, tmp_report_name=None):
    tmp_output_dir  = '{tmpdir}reports-{date}/{tmp_subdir}'.format(
        tmpdir=settings.TMP_DIR,
        date=time.strftime('%Y-%m-%d'),
        tmp_subdir=tmp_subdir,
    )
    tmp_report_file = '{base_dir}{report_name}.pdf'.format(
        base_dir=tmp_output_dir,
        report_name=tmp_report_name or report.get_name(),
    )
    logger.debug("Copying report to temp directory: %s" % tmp_report_file)
    if not os.path.exists(os.path.dirname(tmp_report_file)):
        os.makedirs(os.path.dirname(tmp_report_file))
    shutil.copyfile(
        src=report.get_compiled_dir() + report.get_name() + '.pdf',
        dst=tmp_report_file
    )
    return tmp_output_dir

def copy_sankey_partitions_to_tmp_dir(com_ev, tmp_output_dir):
    sankey_file = com_ev.get_images_dir() + 'sankey-partitions.html'
    if not os.path.exists(sankey_file):
        logger.debug("Cannot copy {ground_truth} Sankey diagram as file doesn't exists: {file}".format(
            ground_truth=com_ev.get_method_id(),
            file=sankey_file,
        ))
        return
    tmp_sankey_file = '{base_dir}{cev}-sankey-partitions.html'.format(
        base_dir=tmp_output_dir,
        cev=com_ev.get_method_id(),
    )
    logger.debug("Copying {ground_truth} Sankey diagram to temp directory: {file}".format(
        ground_truth=com_ev.get_method_id(),
        file=tmp_sankey_file,
    ))
    shutil.copyfile(
        src=sankey_file,
        dst=tmp_sankey_file
    )
