import numpy as np

from ..scoring_method import ScoringMethod
from .. import static_clustering
from .. import settings

class AIC(ScoringMethod):
    """
    Bayesian Information Criterion
    """

    def __init__(self, partition_quality_metric='mod'):
        self._partition_quality_metric = partition_quality_metric

    def get_id(self):
        return '({prefix})'.format(
            prefix=AIC.get_id_prefix(),
        )

    @staticmethod
    def get_id_prefix():
        return 'aic'

    @staticmethod
    def get_scoring_method_title(values):
        return "akaike information criterion"

    def get_limits(self):
        return (None, 0)

    def evaluate_community_evolution(self, community_evolution, dynamic_network=None):
        if not community_evolution.has_partitions():
            return None
        return ScoringMethod.evaluate_community_evolution(self, community_evolution, dynamic_network)

    def evaluate(self, networks_list, segments_partitions, changepoints):
        if len(segments_partitions) == 0:
            return None
        changepoints_with_start_end = [0] + changepoints + [len(networks_list)]
        log_l = 0.0
        for i in xrange(1, len(changepoints_with_start_end)):
            log_l += static_clustering.get_segment_log_l(
                segment_snapshots=networks_list[changepoints_with_start_end[i-1]:changepoints_with_start_end[i]],
                partition=segments_partitions[i-1],
            )
        n_params = sum((p.get_clusters_count() * (p.get_clusters_count() + 1)) / 2 for p in segments_partitions)
        return log_l - n_params


