import numpy as np

from ..scoring_method import ScoringMethod
from .. import static_clustering
from .. import settings
from ..com_evs_similarity_methods import com_evs_similarity_methods_utils

class ComEvsSimilarityAdapter(ScoringMethod):
    """
    An adapter method that computes community evolution score based on its similarity to ground truth
    """

    def __init__(self, com_evs_similarity_method_values, ground_truth_method_id):
        self._com_evs_similarity_method = com_evs_similarity_methods_utils.create_com_evs_similarity_method_by_values(com_evs_similarity_method_values)
        self._ground_truth_method_id = ground_truth_method_id

    def get_id(self):
        return '({prefix})({com_evs_similarity_method_id})({ground_truth_method_id})'.format(
            prefix=ComEvsSimilarityAdapter.get_id_prefix(),
            com_evs_similarity_method_id=self.get_com_evs_similarity_method().get_id(),
            ground_truth_method_id=self.get_ground_truth_method_id(),
        )

    @staticmethod
    def get_id_prefix():
        return 'com_evs_sim'

    @staticmethod
    def get_scoring_method_title(values):
        return "community evolution similarity to {ground_truth_method_id} via {com_evs_similarity_method_id}".format(
            ground_truth_method_id=values[2],
            com_evs_similarity_method_id=values[1],
        )

    def get_limits(self):
        return (0,1)
    def get_com_evs_similarity_method(self):
        return self._com_evs_similarity_method

    def get_ground_truth_method_id(self):
        return self._ground_truth_method_id

    def evaluate_community_evolution(self, community_evolution, dynamic_network=None):
        dyn_network = dynamic_network or community_evolution.get_dynamic_network()
        gt_com_ev = dyn_network.get_ground_truth(self.get_ground_truth_method_id())
        if gt_com_ev is None:
            return None
        return self.get_com_evs_similarity_method().compute_similarity(
            community_evolution,
            gt_com_ev
        )


