import numpy as np

from ..scoring_method import ScoringMethod
from .. import static_clustering
from .. import settings
import scoring_methods_utils
from scipy.stats import gmean

class CombScoringMethod(ScoringMethod):
    """
    Combine results of other methods
    """

    def __init__(self, type, *args):
        self._type = type
        self._params = {}
        vals = []
        if type == 'sum':
            self._params['alpha'] = float(args[0])
            self._params['methods'] = map(scoring_methods_utils.create_scoring_method_by_values, args[1:3])
            vals = [args[0]] + map(str, self._params['methods'])
        elif type == 'gmean':
            self._params['methods'] = scoring_methods_utils.create_scoring_method_by_values(args[0])
            self._params['methods'] = map(scoring_methods_utils.create_scoring_method_by_values, args)
            vals =  map(str, self._params['methods'])

        self._id = ''.join('(%s)' % s for s in ([CombScoringMethod.get_id_prefix(), type] + vals))

    def get_id(self):
        return self._id

    @staticmethod
    def get_id_prefix():
        return 'comb'

    @staticmethod
    def get_scoring_method_title(values):
        comb_type = values[1]
        if comb_type == 'sum':
            return "({method1}) + {alpha}*({method2})".format(
                alpha=float(values[2]),
                method1=scoring_methods_utils.get_scoring_method_title_by_values(values[3]),
                method2=scoring_methods_utils.get_scoring_method_title_by_values(values[4])
            )
        else:
            return "geom mean of [{methods}]".format(
                methods=", ".join(map(scoring_methods_utils.get_scoring_method_title_by_values, values[2:]))
            )

    def get_limits(self):
        if self._type == 'sum':
            return (
                self._params['methods'][0].get_limits()[0]
                + self._params['alpha'] * self._params['methods'][1].get_limits()[0],
                self._params['methods'][0].get_limits()[1]
                + self._params['alpha'] * self._params['methods'][1].get_limits()[1],
            )
        elif self._type == 'gmean':
            return (
                gmean(m.get_limits()[0] for m in self._params['methods']),
                gmean(m.get_limits()[1] for m in self._params['methods']),
            )
        return None

    def evaluate(self, networks_list, segments_partitions, changepoints):
        if self._type == 'sum':
            return self._params['methods'][0].evaluate(networks_list, segments_partitions, changepoints) + \
                   self._params['alpha'] * self._params['methods'][1].evaluate(networks_list, segments_partitions, changepoints)
        elif self._type == 'gmean':
            return gmean([m.evaluate(networks_list, segments_partitions, changepoints) for m in self._params['methods']])
        return None


