from avg_segment_partition_quality import  AvgSegmentPartitionQuality
from avg_snapshot_partition_quality import AvgSnapshotPartitionQuality
from comb_scoring_method import CombScoringMethod
from segments_sizes_quality import SegmentsSizesQuality
from com_evs_similarity_adapter import ComEvsSimilarityAdapter
from classification_result_adapter import ClassificationResultAdapter
from bic import BIC
from aic import AIC
from ..utils import parse_parentheses

def create_scoring_method_by_id(scoring_method_id):
    values = parse_parentheses(scoring_method_id)
    return create_scoring_method_by_values(values)

def create_scoring_method_by_values(values):
    if values[0] == AvgSnapshotPartitionQuality.get_id_prefix():
        return AvgSnapshotPartitionQuality(
            partition_quality_metric=values[1]
        )
    elif values[0] == AvgSegmentPartitionQuality.get_id_prefix():
        return AvgSegmentPartitionQuality(
            consensus_partition_quality_metric=values[1]
        )
    elif values[0] == SegmentsSizesQuality.get_id_prefix():
        sgmq_type = values[1]
        if sgmq_type == 'exp':
            return SegmentsSizesQuality(
                type=sgmq_type,
                eps=float(values[2])
            )
        else:
            return SegmentsSizesQuality(
                type=sgmq_type
            )
    elif values[0] == ComEvsSimilarityAdapter.get_id_prefix():
        return ComEvsSimilarityAdapter(
            com_evs_similarity_method_values=values[1],
            ground_truth_method_id=values[2],
        )
    elif values[0] == ClassificationResultAdapter.get_id_prefix():
        return ClassificationResultAdapter(
            metric=values[1],
            ground_truth_method_id=values[2],
        )
    elif values[0] == BIC.get_id_prefix():
        return BIC()
    elif values[0] == AIC.get_id_prefix():
        return AIC()
    elif values[0] == CombScoringMethod.get_id_prefix():
        return CombScoringMethod(values[1],*values[2:])
    raise Exception("Unknown scoring method id: %s" % values)

def get_scoring_method_title(scoring_method_id):
    values = parse_parentheses(scoring_method_id)
    return get_scoring_method_title_by_values(values)

def get_scoring_method_class_by_id(scoring_method_id):
    values = parse_parentheses(scoring_method_id)
    return get_scoring_method_class_by_prefix(values[0])

def get_scoring_method_class_by_prefix(prefix):
    if prefix == AvgSnapshotPartitionQuality.get_id_prefix():
        return AvgSnapshotPartitionQuality
    elif prefix == AvgSegmentPartitionQuality.get_id_prefix():
        return AvgSegmentPartitionQuality
    elif prefix == SegmentsSizesQuality.get_id_prefix():
        return SegmentsSizesQuality
    elif prefix == ComEvsSimilarityAdapter.get_id_prefix():
        return ComEvsSimilarityAdapter
    elif prefix == ClassificationResultAdapter.get_id_prefix():
        return ClassificationResultAdapter
    elif prefix == BIC.get_id_prefix():
        return BIC
    elif prefix == AIC.get_id_prefix():
        return AIC
    elif prefix == CombScoringMethod.get_id_prefix():
        return CombScoringMethod
    raise Exception("Unknown scoring method prefix: %s" % prefix)

def get_scoring_method_title_by_values(values):
    return get_scoring_method_class_by_prefix(values[0]).get_scoring_method_title(values)
