import numpy as np

from ..scoring_method import ScoringMethod
from .. import static_clustering
from .. import settings
from scipy.stats import gmean

class SegmentsSizesQuality(ScoringMethod):
    """
    Evaluate segmentation quality based on segments sizes balance
    """

    def __init__(self, type, **kwargs):
        self._type = type
        self._params = {}
        if type == 'exp':
            self._params['eps'] = kwargs.get('eps')

    def get_id(self):
        return '({prefix})({type}){params}'.format(
            prefix=SegmentsSizesQuality.get_id_prefix(),
            type=self._type,
            params=''.join('(%s)' % self._params[k] for k in sorted(self._params.keys()))
        )

    @staticmethod
    def get_id_prefix():
        return 'sgmq'

    @staticmethod
    def get_scoring_method_title(values):
        sgmq_type = values[1]
        if sgmq_type == 'exp':
            return "{type} of segment sizes with eps={eps}".format(
                type=sgmq_type,
                eps=float(values[2])
            )
        elif sgmq_type == 'num':
            return "$l$" # "Number of segments"
        else:
            return "{type} of segment sizes".format(
                type=sgmq_type,
            )

    def get_limits(self):
        if self.get_type() == 'exp':
            return (0, 1)
        elif self.get_type() == 'num':
            return (0, None)
        elif self.get_type() == 'gmean':
            return (0, None)

    def get_type(self):
        return self._type

    def get_param(self, key):
        return self._params.get(key)

    def evaluate(self, networks_list, segments_partitions, changepoints):
        changepoints_with_start_end = [0] + changepoints + [len(networks_list)]
        segments_sizes = []
        for i in xrange(1, len(changepoints_with_start_end)):
            segments_sizes.append(
                changepoints_with_start_end[i] - changepoints_with_start_end[i-1]
            )
        if self.get_type() == 'exp':
            return np.exp(
                (-1 * (max(segments_sizes) - min(segments_sizes))) / self.get_param('eps')
            )
        elif self.get_type() == 'num':
            return len(segments_sizes)
        elif self.get_type() == 'gmean':
            return gmean(segments_sizes)



