import os
import utils
from multiprocessing import cpu_count


BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(utils.__file__)))).replace('\\','/') + '/'
NETWORKS_BASE_DIR = BASE_DIR + 'networks/'
NETWORKS_RAW_DATA_DIR = NETWORKS_BASE_DIR + '_raw/'
if not utils.is_remote_server(): # a hack to avoid issue with long paths for Windows
    NETWORKS_BASE_DIR = 'C:/dc/'

TEMPLATES_DIR = BASE_DIR + 'templates/'
TMP_DIR = BASE_DIR + 'tmp/'
LOGS_DIR = BASE_DIR + 'logs/'
REPORTS_DIR = BASE_DIR + 'reports/'
EVAL_DIR = NETWORKS_BASE_DIR + '_eval/' # network-independent evaluation


#---------------SYNTHETIC NETWORKS--------------------------
SYNTHETIC_NETWORK_ID_GLOBAL_PREFIX = 'synthetic/'
SYNTHETIC_DATA_DIR = NETWORKS_BASE_DIR + SYNTHETIC_NETWORK_ID_GLOBAL_PREFIX + '_data/'
# where random instances of synthetically generated community evolutions are located
SYNTHETIC_SEGMENTS_PARTITIONS_DIR = SYNTHETIC_DATA_DIR + 'segments_partitions/'
# where community evolution steps are located (e.g., merge/split community)
SYNTHETIC_SEGMENTS_PARTITIONS_HISTORIES_DIR = SYNTHETIC_DATA_DIR + 'segments_partitions_histories/'

#---------------IDS-----------------------------------------
METHOD_ID_SEPARATOR = '-'
CONSENSUS_CLUSTERING_ID_SEPARATOR = '^'
SYNTHETIC_COMMUNITY_EVOLUTION_HISTORY_ID_SEPARATOR = '_'
SYNTHETIC_COMMUNITY_EVOLUTION_HISTORY_TYPE_SEPARATOR = '.'
SYNTHETIC_NETWORK_ID_SEPARATOR = '-'
COMMUNITY_EVOLUTION_RECORD_SEPARATOR = ';'
COMMUNITY_EVOLUTION_RECORD_CLUSTERS_LIST_SEPARATOR = ','

#---------------PARALLEL------------------------------------
MAX_PROCESSES_COUNT = cpu_count()


#---------------DRAW----------------------------------------
SANKEY_WEBPAGE_TEMPLATE = TEMPLATES_DIR + 'sankey/sankey_template_one_file.html'
CHART_DPI = 300
CHART_EXT = 'pdf'
# or ['#348ABD', '#7A68A6', '#A60628', '#467821', '#CF4457', '#188487', '#E24A33']
# or [
#     'blue', 'green', 'red', 'orange', 'magenta', 'cyan', 'brown',
#     'silver', 'deeppink', 'limegreen', 'darkgoldenrod', 'violet',
# ]
LINE_COLORS = [
    '#5DA5DA', # blue
    '#FAA43A', # orange
    '#60BD68', # green
    '#F15854', # red
    '#B276B2', # purple
    '#8C564B', # dark brown
    '#F17CB0', # pink
    '#4D4D4D', # gray
    '#9EDAE5', # light blue
    '#B2912F', # light brown
    '#DECF3F', # yellow
    '#9ADBAD', # light green
]
LINE_MARKERS = [
    'o',
    's',
    'D',
    '^',
    'v',
    '<',
    '>',
    'p',  'h', '8', 'd', 'H', '+',
]

#---------------REPORTS-------------------------------------
LATEX_TEMPLATE_FILE = TEMPLATES_DIR + 'reports/report.tex'
MAKEFILE_TEMPLATE_FILE = TEMPLATES_DIR + 'reports/makefile'
REPORT_AUTHOR = 'Yuriy Hulovatyy'


# default metrics
DEFAULT_PARTITION_QUALITY_METRICS = ['mod', 'cond_q', 'log_l']
DEFAULT_PARTITION_SIMILARITY_METRICS = ['ami', 'nmi', 'ari', 'hom', 'com', 'vm']
DEFAULT_PARTITION_SIMILARITY_AFFINITY_METRICS = ['sil', 'avg_cl_sim']
DEFAULT_SCORING_METHODS_IDS = [
    '(avg_sn)(mod)',
    #'(avg_sn)(log_l)',
    #'(bic)',
    #'(avg_sn)(cond_q)',
    #'(avg_sn)(norm_cut_q)',
    #'(avg_sn)(avg_odf_q)',
    # '(avg_sn)(perf_q)',
    # '(avg_sn)(cov_q)',
    # '(com_evs_sim)((gl_p)(ami))(segment_ground_truth)',
    #'(aic)',
    #'(bic)',
]
DEFAULT_COM_EVS_SIMILARITY_METHODS_IDS_LIST = [
     '(gl_p)(ami)',
    #'(gl_p)(nmi)',
    #'(gl_p)(ari)',
    #'(gl_p)(vm)',
    # '(avg_sn_p)(ami)',
    # '(avg_sn_p)(hom)',
    # '(avg_sn_p)(com)',
    # '(avg_sn_p)(vm)',
    #'(avg_sn_p)(sil)',
    #'(avg_sn_p)(avg_cl_sim)',
]
DEFAULT_COM_EVS_SIMILARITY_METHODS_IDS_DICT = {
    'global': [
        '(gl_p)(ami)',
        #'(gl_p)(nmi)',
        #'(gl_p)(ari)',
        #'(gl_p)(vm)',
    ],
    # 'partition': [
    #     '(avg_sn_p)(ami)',
    #     # '(avg_sn_p)(ari)',
    #     # '(avg_sn_p)(nmi)',
    #     '(avg_sn_p)(hom)',
    #     '(avg_sn_p)(com)',
    #     # '(avg_sn_p)(vm)',
    #     '(avg_sn_p)(sil)',
    #     '(avg_sn_p)(avg_cl_sim)',
    # ],
    # 'time': [
    #     '(avg_seg_t)(ami)',
    #     # '(avg_seg_t)(nmi)',
    #     # '(avg_seg_t)(ari)',
    #     # '(avg_seg_t)(vm)',
    # ],
}
DEFAULT_COM_EVS_SIMILARITY_AFFINITY_METHODS_IDS_DICT = {
    'partition': [
        '(avg_sn_p)(sil)',
    ],
}
DEFAULT_COM_EVS_SCATTER_PLOT_METHODS_IDS_PAIRS = [
    ('(avg_seg_t)(nmi)', '(avg_sn_p)(nmi)'),
   # ('(avg_seg_t)(ami)', '(avg_sn_p)(ami)'),
#    ('(avg_seg_t)(vm)', '(avg_sn_p)(vm)'),
]
