import os, re, time, sys
import numpy as np
import string
import random
import hashlib
from distutils.util import strtobool

def is_remote_server():
    return os.name != 'nt'

def id_generator(size=5, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))

def hash_string(str, size=5):
    return hashlib.md5(str).hexdigest()[:size]

def user_yes_no_prompt(message):
    sys.stdout.write('%s [y/n]\n' % message)
    while True:
        try:
            return strtobool(raw_input().lower())
        except ValueError:
            sys.stdout.write('Please respond with \'y\' or \'n\'.\n')

def cmp_str_int(str1,str2):
    '''
    Compare strings of the format 'str_int'.
    :param str1:
    :param str2:
    :return:
    '''
    s1,i1 = '_'.join(str1.split('_')[:-1]),int(re.sub(r'\D', '', str1.split('_')[-1]))
    s2,i2 = '_'.join(str2.split('_')[:-1]),int(re.sub(r'\D', '', str2.split('_')[-1]))
    if s1!=s2:
        return cmp(s1,s2)
    return cmp(i1,i2)

def get_file_list_from_dir(dir, ext=None, full_path=True, include_dirs=False, recursive=False):
    res = []
    for path, subdirs, files in os.walk(dir, ):
        for name in files:
            if ext is None or ('.'+ext) in name:
                res.append(os.path.join(path, name) if full_path else name)
        if include_dirs:
            for name in subdirs:
                res.append(os.path.join(path, name) if full_path else name)
        if not recursive:
            break
    return res

def get_subdir_list_from_dir(dir, full_path=True):
    return [
        os.path.join(dir, name) if full_path else name
            for name in (os.listdir(dir) if os.path.exists(dir) else [])
            if os.path.isdir(os.path.join(dir, name))
    ]

def clean_dir(dir, pattern):
    """
    Remove all files matching a pattern
    :param dir:
    :param pattern:
    :return:
    """

    for f in os.listdir(dir):
        if re.search(pattern, f):
            os.remove(os.path.join(dir, f))

def sec_to_time(sec):
    """
    Convert seconds to time string
    """
    days = sec // 86400
    sec -= 86400*days

    hrs = sec // 3600
    sec -= 3600*hrs

    mins = sec // 60
    sec -= 60*mins
    return '%d:%d:%d:%.3f' % (days, hrs, mins, sec)

def timestamp_to_time(unix_timestamp, formatString = "%Y-%m-%d"):
    return time.strftime(formatString, time.localtime(unix_timestamp))


def get_intersection(*args):
    return reduce(set.intersection, map(set, args))

def concatenate_files(filenames,outputfile):
    with open(outputfile, 'w') as outfile:
        for fname in filenames:
            with open(fname) as infile:
                for line in infile:
                    outfile.write(line)

def shuffle_dict(input_dict):
    dict_values = input_dict.values()
    np.random.shuffle(dict_values)
    return dict(zip(input_dict.keys(), dict_values))

def remove_duplicates_stable(seq):
    """
    Remove duplicated without changin elements' order
    :param seq:
    :return:
    """
    seen = set()
    seen_add = seen.add
    return [ x for x in seq if not (x in seen or seen_add(x))]

def parse_user_args_string_into_values_list(s):
    temp_list = s.split(',')
    res = []
    for s in temp_list:
        if '[' in s:
            ind_start = s.index('[')
            ind_end = s.index(']')
            s_split = s[ind_start + 1: ind_end]

            values = []
            temp_values = s_split.split('|')
            # e.g. aaa-[1-10]-aaa => aaa-1-aaa, aaa-2-aaa, ..., aaa-10-aaa
            for v in temp_values:
                if '-' in v and all(x.isdigit() for x in v.split('-')):
                    values_boundary = map(int, v.split('-'))
                    values += range(values_boundary[0], values_boundary[1] + 1)
                else: # aaa-[b|c]-aaa => aaa-b-aaa, aaa-c-aaa
                    values.append(v)

            for v in values:
                for suffix in parse_user_args_string_into_values_list(s[ind_end + 1:]):
                    res.append(
                        s[:ind_start] + str(v) + suffix
                    )
        else:
            res.append(s)

    return res

def parse_parentheses(xs):
    stack = [[]]
    for x in xs:
        if x == '(':
            stack[-1].append([])
            stack.append(stack[-1][-1])
        elif x == ')':
            stack.pop()
            if not stack:
                raise ValueError("Error: opening bracket is missing")
        else:
            if len(stack[-1]) > 0:
                stack[-1][0] += x
            else:
                stack[-1].append(x)
    if len(stack) > 1:
        raise ValueError("Error: closing bracket is missing")

    # convert [['a'],[['b'],['c']]] to ['a',['b','c']]
    def _rec_list_to_str(l):
        rec_res = []
        for x in l:
            if isinstance(x, str):
                rec_res.append(x)
            else:
                rec_res.append(_rec_list_to_str(x))

        if len(rec_res) == 1:
            return rec_res[0]
        return rec_res

    raw_res = stack.pop()
    res = []
    for x in raw_res:
        res.append(_rec_list_to_str(x))
    return res
