import argparse
from core.log import logger, log_func_progress
from core.utils import parse_user_args_string_into_values_list
from core.real_world_networks import (
    enron,
    amd_hope,
    hypertext,
    high_school,
    reality_mining,
    correlates_of_war,
    voteview,
    trade,
)

@log_func_progress
def main():
    parser = argparse.ArgumentParser(description="Create networks from real-world data")
    parser.add_argument("networkname", help="Network name")
    args = parser.parse_args()

    network_names_list = parse_user_args_string_into_values_list(args.networkname)
    for i,network_name in enumerate(network_names_list):
        logger.info("%s/%s. Preprocessing real-world network '%s'" % (i+1, len(network_names_list), network_name))
        if network_name.startswith("enron"):
            values = network_name.split("-")
            enron.create_dynamic_network(
                delta_t=values[1],
                threshold=int(values[2]),
            )
        elif network_name.startswith("amd_hope"):
            values = network_name.split("-")
            network_type = values[1]
            if network_type == 'attend':
                amd_hope.create_dynamic_network(
                    network_type=network_type,
                    min_person_talks=int(values[2]),
                    min_talk_persons=int(values[3]),
                    max_talk_persons=int(values[4]),
                    delta_t=values[5],
                    threshold=int(values[6]),
                )
            else:
                amd_hope.create_dynamic_network(
                    network_type=network_type,
                    min_norm_time_in_area=float(values[2]),
                    delta_t=values[3],
                    threshold=int(values[4]),
                )
        elif network_name.startswith("hypertext"):
            values = network_name.split("-")
            hypertext.create_dynamic_network(
                delta_t=values[1],
                threshold=int(values[2]),
            )
        elif network_name.startswith("high_school"):
            values = network_name.split("-")
            high_school.create_dynamic_network(
                delta_t=values[1],
                threshold=int(values[2]),
            )
        elif network_name.startswith("reality_mining"):
            values = network_name.split("-")
            reality_mining.create_dynamic_network(
                delta_t=values[1],
                threshold=int(values[2]),
            )
        elif network_name.startswith("cow"):
            values = network_name.split("-")
            correlates_of_war.create_dynamic_network(
                network_type=values[1],
                network_sub_type=values[2],
                delta_t=values[3],
                threshold=int(values[4]),
                min_year=int(values[5]) if len(values)>5 else None,
                max_year=int(values[6]) if len(values)>6 else None
            )
        elif network_name.startswith("voteview"):
            values = network_name.split("-")
            voteview.create_dynamic_network(
                delta_t=values[1],
                threshold=float(values[2]),
            )
        elif network_name.startswith("trade"):
            trade.create_dynamic_network()
if __name__ == '__main__':
    main()